# Code for lake acidity data

library(assist)
library(lattice)

data(acid)
acid$x21 <- acid$x1
acid$x22 <- acid$x2
acid$x1 <- acid$t1

# fit a TPS for x1
acid.tp.fit1 <- ssr(ph~x1, rk=tp(x1), data=acid)
summary(acid.tp.fit1)
anova(acid.tp.fit1)

grid1 <- seq(min(acid$x1),max(acid$x1),len=100)
acid.tp.pred1 <- predict(acid.tp.fit1, data.frame(x1=grid1))

# fit a TPS for x1 with spherical spatial correlation	
acid.tp.fit2 <- update(acid.tp.fit1, spar="m",
  corr=corExp(form=~x21+x22,nugget=T))
summary(acid.tp.fit2)
acid.tp.pred2 <- predict(acid.tp.fit2, data.frame(x1=grid1))

# fit an SS ANOVA model for x1 and x2
acid.ssanova.fit <- ssr(ph~x1+x21+x22, 
  rk=list(tp(x1), tp(list(x21,x22))),
  data=acid, spar="m")
summary(acid.ssanova.fit)

acid.ssanova.pred1 <- predict(acid.ssanova.fit, 
  data.frame(x1=grid1,x21=0,x22=0),
  terms=c(1,1,0,0,1,0))
grid2 <- expand.grid(x1=0,
           x21=seq(min(acid$x21), max(acid$x21), len=20),
           x22=seq(min(acid$x22), max(acid$x22), len=20))
acid.ssanova.pred2 <- predict(acid.ssanova.fit, grid2, 
  terms=c(0,0,1,1,0,1))

# xyplots need to be generated separately
postscript(file="acid1.ps",height=2.2*1.5,width=4.45*1.5,horizontal=F,
  pointsize=8)
trellis.par.set("axis.line", list(col="black"))

tmp1 <- data.frame(x=acid$x1, y=acid$ph)
tmp2 <- data.frame(x=grid1, y=acid.tp.pred1$fit)
tmp3 <- data.frame(x=grid1, y=acid.tp.pred2$fit)
tmp4 <- data.frame(x=grid1, y=acid.ssanova.pred1$fit)
tmp1$rowType <- rep("raw", nrow(tmp1))
tmp2$rowType <- rep("tp1", nrow(tmp2))
tmp3$rowType <- rep("tp2", nrow(tmp3))
tmp4$rowType <- rep("ssanova", nrow(tmp4))
all <- rbind(tmp1, tmp2, tmp3, tmp4)
p1 <- xyplot(y~x, data=all, subscripts=T, groups=all$rowType,
       xlab=list("calcium(log10 mg/L)",cex=.75), 
       ylab=list("pH",cex=.75), scale=list(tck=c(1,0),cex=.6),
       panel=function(x,y,subscripts,groups,...){
             raw <- groups[subscripts]=="raw"
             tp1 <- groups[subscripts]=="tp1"
             tp2 <- groups[subscripts]=="tp2"
             ssanova <- groups[subscripts]=="ssanova"
             panel.xyplot(x[raw],y[raw],type="p",cex=.4, col=gray(0:8/8)[5])
             llines(x[tp1],y[tp1])
             llines(x[tp2],y[tp2], lty=2)
             llines(x[ssanova],y[ssanova], lty=3)})
print(p1, split=c(1,1,2,1), position=c(.05,.05,.95,.95), more=T)

trellis.par.set("axis.line", list(col="transparent"))
p2 <- wireframe(acid.ssanova.pred2$fit~grid2$x21*grid2$x22, 
 screen=list(z=-25,x=-75), xlab=list("latitude",cex=.7,rot=-8),
 ylab=list("longitude",cex=.7,rot=50), zlab=list("pH",cex=.7),
 scales=list(arrows=F, cex=.6, col="black", tck=1))
print(p2, split = c(2,1,2,1))

dev.off()


# fit a thin-plate spline with exponential spatial correlation
temp <- ssr(ph~x1+x21+x22, rk=tp(x1), data=acid,
	    corr=corExp(form=~x21+x22, nugget=T), spar="m")	
tau <- coef(temp$cor.est, F)
D <- corMatrix(Initialize(corExp(tau[1],form=~x21+x22), data=acid))
Z <- chol.new(D)

# fit a SLM model with the estimated correlation matrix
acid.slm.fit <- slm(ph~x1+x21+x22, rk=tp(x1), data=acid,
	            random=list(pdIdent(~Z-1)))
summary(acid.slm.fit)

grid3 <- data.frame(x1=grid1, x21=min(acid$x21), x22=min(acid$x22))
acid.slm.x1.p <- intervals(acid.slm.fit, newdata=grid3, terms=c(1,1,0,0,1))
grid4 <- expand.grid(x21=seq(min(acid$x21)-.001,max(acid$x21)+.001, len=20),
	             x22=seq(min(acid$x22)-.001,max(acid$x22)+.001, len=20))
newdata <- data.frame(z1=c(acid$x21,grid4$x21), z2=c(acid$x22,grid4$x22))
RD <- corMatrix(Initialize(corExp(tau[1], form=~z1+z2), data=newdata))
R <- RD[(length(acid$x21)+1):length(newdata$z1),1:length(acid$x21)] 
u.new <- R%*%t(solve(Z))%*%as.vector(acid.slm.fit$lme.obj$coef$random[[2]])
acid.slm.x2.p <- acid.slm.fit$lme.obj$coef$fixed[3]*grid4$x21+
                 acid.slm.fit$lme.obj$coef$fixed[4]*grid4$x22+u.new


postscript(file="acid2.ps",height=2.2*1.5,width=4.45*1.5,horizontal=F,
  pointsize=8)
trellis.par.set("axis.line", list(col="black"))

tmp1 <- data.frame(x=acid$x1, y=acid$ph)
tmp2 <- data.frame(x=grid1, y=acid.slm.x1.p$fit)
tmp1$rowType <- rep("raw", nrow(tmp1))
tmp2$rowType <- rep("slm", nrow(tmp2))
all <- rbind(tmp1, tmp2)
p1 <- xyplot(y~x, data=all, subscripts=T, groups=all$rowType,
       xlab=list("calcium(log10 mg/L)",cex=.75), 
       ylab=list("pH",cex=.75), scale=list(tck=c(1,0),cex=.6),
       panel=function(x,y,subscripts,groups,...){
             raw <- groups[subscripts]=="raw"
             slm <- groups[subscripts]=="slm"
             ssanova <- groups[subscripts]=="ssanova"
             panel.xyplot(x[raw],y[raw],type="p",cex=.4, col=gray(0:8/8)[5])
             llines(x[slm],y[slm])})
print(p1, split=c(1,1,2,1), position=c(.05,.05,.95,.95), more=T)

trellis.par.set("axis.line", list(col="transparent"))
p2 <- wireframe(acid.slm.x2.p~grid4$x21*grid4$x22, 
 screen=list(z=-25,x=-75), xlab=list("latitude",cex=.7,rot=-8),
 ylab=list("longitude",cex=.7,rot=50), zlab=list("pH",cex=.7),
 scales=list(arrows=F, cex=.6, col="black", tck=1))
print(p2, split = c(2,1,2,1))

dev.off()

