# Code for air quality data

library(assist)
air <- na.omit(airquality)
attach(air)

y <- Ozone^(1/3)
x1 <- Wind
x2 <- Temp
x3 <- ident(Solar.R)

# single index model
air.snr.1 <- snr(y~f1(b1*x1+b2*x2+sqrt(1-b1^2-b2^2)*x3),
                 func=f1(u)~list(~u, rk=tp(u)), 
                 params=list(b1+b2~1), spar="m",
                 start=list(params=c(-0.8,.5)), 
                 control=list(maxit.out=50),
                 verbose=T)

# partial single index model
air.snr.2 <- snr(y~f2(b1*x1+sqrt(1-b1^2)*x2)+b2*x3,
                 func=f2(u)~list(~u, rk=tp(u)), 
                 params=list(b1+b2~1), spar="m",
                 start=list(params=c(-0.8,1.3)), 
                 verbose=T)

# additive single index model
air.snr.3 <- snr(y~f3(a1*x1+sqrt(1-a1^2)*x2)+f4(x3),
            func=list(f3(u)~list(~u, rk=tp(u)), 
                      f4(v)~list(~v-1, rk=cubic(v))), 
            params=list(a1~1), spar="m",
            start=list(params=c(-0.8)), verbose=T)

# compute GCV, AIC and BIC
n <- 111
rss <- c(sum((y-air.snr.1$fitted)**2),
         sum((y-air.snr.2$fitted)**2),
         sum((y-air.snr.3$fitted)**2))/n
df <- c(air.snr.1$df$f+air.snr.1$df$para,
        air.snr.2$df$f+air.snr.2$df$para,
        air.snr.3$df$f+air.snr.3$df$para)
gcv <- rss/(1-df/n)**2
aic <- n*log(rss)+2*df
bic <- n*log(rss)+log(n)*df
print(round(rbind(aic, bic, gcv),4))

# plot
indx1 <- air.snr.1$coef[1]*x1+air.snr.1$coef[2]*x2+
 sqrt(1-air.snr.1$coef[1]^2-air.snr.1$coef[2]^2)*x3
indx2 <- air.snr.2$coef[1]*x1+sqrt(1-air.snr.2$coef[1]^2)*x2
indx3 <- air.snr.3$coef[1]*x1+sqrt(1-air.snr.3$coef[1]^2)*x2
grid1 <- seq(min(indx1), max(indx1),len=100)
grid2 <- seq(min(indx2,indx3), max(indx2,indx3),len=100)
grid3 <- seq(0,1,len=100)
p1 <- intervals(air.snr.1,newdata=data.frame(u=grid1))
p2 <- intervals(air.snr.2,newdata=data.frame(u=grid2))
p3 <- intervals(air.snr.3,newdata=data.frame(u=grid2,v=grid3))
pr1 <- y-air.snr.3$funcFitted[112:222]
pr2 <- y-air.snr.3$fitted+air.snr.3$funcFitted[112:222]

postscript(file="air.ps",height=2,width=4.4,horizontal=F,pointsize=10)
par(mfrow=c(1,3),mgp=c(2,1,0), mar=c(3,3,1,1)+.1) 

yr <- c(-1,5.5)
plot(grid1, p1$f1$fit[,1],xlab="index",ylab=expression(f[1]),ylim=yr,type="n")
polygon(c(grid1,rev(grid1)),
        c(p1$f1$fit[,1]-2*p1$f1$pstd,rev(p1$f1$fit[,1]+2*p1$f1$pstd)),
        col="lightgray", border=NA)                  
lines(grid1, p1$f$fit[,1])  
points(indx1, y, pch=".", cex=1.8)    
mtext("(a)")

plot(grid2, p2$f2$fit[,1],xlab="index",ylim=yr,type="n",
 ylab=expression(paste(f[2]," & ",f[3])))
polygon(c(grid2,rev(grid2)),
        c(p3$f3$fit[,1]-2*p3$f3$pstd,rev(p3$f3$fit[,1]+2*p3$f3$pstd)),
        col="lightgray", border=NA)                  
lines(grid2, p2$f2$fit[,1]-mean(p2$f2$fit[,1])+mean(p3$f3$fit[,1]),lty=2)
lines(grid2, p3$f3$fit[,1])  
points(indx3, pr1-mean(pr1)+mean(p3$f3$fit[,1]), pch=".", cex=1.8)     
mtext("(b)")

xgrid3 <- grid3*(diff(range(Solar.R)))
plot(xgrid3, p3$f4$fit[,1],xlab="radiation",ylab=expression(f[4]),ylim=yr,type="n")
polygon(c(xgrid3,rev(xgrid3)),
        c(p3$f4$fit[,1]-2*p3$f4$pstd,rev(p3$f4$fit[,1]+2*p3$f4$pstd)),
        col="lightgray", border=NA)                  
lines(xgrid3, air.snr.2$coef[2]*grid3-mean(air.snr.2$coef[2]*grid3)+mean(p3$f4$fit[,1]),lty=2)
lines(xgrid3, p3$f4$fit[,1])  
points(Solar.R, pr2-mean(pr2)+mean(p3$f4$fit[,1]), pch=".", cex=1.8)        
mtext("(c)")

dev.off()                     




