# Code for Arosa ozone data

library(assist)
library(lattice)
data(Arosa)
Arosa$x <- (Arosa$month-0.5)/12
attach(Arosa)

# fit a periodic spline	
arosa.per.fit <- ssr(thick~1, rk=periodic(x)) 
grid <- data.frame(x=seq(0,1,length=50))
arosa.per.pred <- predict(arosa.per.fit, newdata=grid)
postscript(file="arosa1.ps",height=2.4,width=3.5,
  horizontal=F,pointsize=8)
par(mgp=c(2,1,0), mar=c(3,3,1,1)+0.1)
plot(Arosa$x, Arosa$thick, xlim=c(0,1), xlab="month",
  ylab="thickness",axes=F,type="n")
box()
axis(2)
axis(1,at=sort(unique(x)), label=as.character(1:12),cex=0.8)
polygon(c(grid$x,rev(grid$x)), 
        c(arosa.per.pred$fit-1.96*arosa.per.pred$pstd, 
          rev(arosa.per.pred$fit+1.96*arosa.per.pred$pstd)),
          col=gray(0:8/8)[8], border=NA)
points(Arosa$x, Arosa$thick, pch=".", cex=.8)
lines(grid$x, arosa.per.pred$fit)
dev.off()

# fit a partial spline model	
arosa.ps.fit <- ssr(thick~sin(2*pi*x)+cos(2*pi*x), 
                  rk=periodic(x), data=Arosa)
summary(arosa.ps.fit)
anova(arosa.ps.fit, simu.size=500)

# fit an l-spline model	
arosa.ls.fit <- ssr(thick~sin(2*pi*x)+cos(2*pi*x), 
                  rk=lspline(x,type="sine1"))
summary(arosa.ls.fit)
anova(arosa.ls.fit,simu.size=500)

grid <- data.frame(x=seq(.5/12,11.5/12,length=50))
arosa.per.pred <- predict(arosa.per.fit, grid,
  terms=matrix(c(1,0,0,1,1,1),nrow=3,byrow=T))
arosa.ps.pred <- predict(arosa.ps.fit, grid,
  terms=matrix(c(1,1,1,0,0,0,0,1,1,1,1,1),nrow=3,byrow=T))
arosa.ls.pred <- predict(arosa.ls.fit,grid,
  terms=matrix(c(1,1,1,0,0,0,0,1,1,1,1,1),nrow=3,byrow=T))
ci.per <- data.frame(fit=as.vector(arosa.per.pred[[1]]), 
  pstd=as.vector(arosa.per.pred[[2]]))
ci.per$upper <- ci.per$fit+1.96*ci.per$pstd
ci.per$lower <- ci.per$fit-1.96*ci.per$pstd
ci.per$pstd <- NULL
ci.ps <- data.frame(fit=as.vector(arosa.ps.pred[[1]]), 
  pstd=as.vector(arosa.ps.pred[[2]]))
ci.ps$upper <- ci.ps$fit+1.96*ci.ps$pstd
ci.ps$lower <- ci.ps$fit-1.96*ci.ps$pstd
ci.ps$pstd <- NULL
ci.ls <- data.frame(fit=as.vector(arosa.ls.pred[[1]]), 
  pstd=as.vector(arosa.ls.pred[[2]]))
ci.ls$upper <- ci.ls$fit+1.96*ci.ls$pstd
ci.ls$lower <- ci.ls$fit-1.96*ci.ls$pstd
ci.ls$pstd <- NULL
fit.data <- data.frame(thick=c(ci.per$fit, ci.ps$fit, ci.ls$fit), 
   x=rep(12*grid$x+.5, 9), 
   grp=rep(rep(c("parametric", "smooth", "overall"), rep(50, 3)), 3), 
   type=rep(c("periodic spline","partial spline","L-spline"), rep(150, 3)))
low.data <- data.frame(thick=c(ci.per$lower, ci.ps$lower, ci.ls$lower), 
   x=rep(12*grid$x+.5, 9), 
   grp=rep(rep(c("parametric", "smooth", "overall"), rep(50, 3)), 3), 
   type=rep(c("periodic spline","partial spline","L-spline"), rep(150, 3)))
up.data <- data.frame(thick=c(ci.per$up, ci.ps$up, ci.ls$up), 
   x=rep(12*grid$x+.5, 9), 
   grp=rep(rep(c("parametric", "smooth", "overall"), rep(50, 3)), 3), 
   type=rep(c("periodic spline","partial spline","L-spline"), rep(150, 3)))
fit.data$type <- factor(fit.data$type, 
   levels=c("L-spline","partial spline","periodic spline"))

# the trellis needs to be created manually
trellis.device(postscript, file="arosa2.ps", width=5.72, 
               height=6.5, horizontal=F) 
xyplot2(thick~x|grp*type, data=list(fit.data, up.data, low.data),
  par.strip.text = list(cex = 0.75),xlab="month",ylab="thickness")
dev.off()

# fit a partial spline with both month and year effects
Arosa$x1 <- (Arosa$month-0.5)/12
Arosa$x2 <- (Arosa$year-1)/45
arosa.ps.fit2 <- ssr(thick~sin(2*pi*x1)+cos(2*pi*x1)+x2,  
                     rk=cubic(x2), data=Arosa)
summary(arosa.ps.fit2)

m1 <- sapply(split(Arosa$thick,Arosa$month),mean)-mean(Arosa$thick)
m2 <- sapply(split(Arosa$thick,Arosa$year),mean)-mean(Arosa$thick)
grid1 <- data.frame(x1=seq(.5/12,11.5/12,len=100), x2=.5)
grid2 <- data.frame(x1=.5, x2=seq(0,1,len=100))
arosa.ps.pred21 <- predict(arosa.ps.fit2,grid1,terms=c(0,1,1,0,0))
arosa.ps.pred22 <- predict(arosa.ps.fit2,grid2,terms=c(0,0,0,1,1))
postscript(file="arosa3.ps",height=2.1,width=4.4,
           horizontal=F,pointsize=6)
par(mfrow=c(1,2), mgp=c(2,1,0), mar=c(3,3,3,1))
plot(sort(unique(Arosa$x)),m1,xlim=c(0,1),ylim=c(-55,50),
     xlab="month",ylab="thickness",axes=F,type="n")
box()
axis(2,cex=.3)
axis(1,at=sort(unique(Arosa$x)),label=as.character(1:12),cex=0.3)
polygon(c(grid1$x1,rev(grid1$x1)),
        c(arosa.ps.pred21$fit-1.96*arosa.ps.pred21$pstd,
          rev(arosa.ps.pred21$fit+1.96*arosa.ps.pred21$pstd)),
        col="lightgray",border=NA)
points(sort(unique(Arosa$x)),m1,pch="o",cex=0.7)
lines(grid1$x1,arosa.ps.pred21$fit)
mtext("Main effect of month",cex=1.2)

plot(unique(Arosa$x2),m2,xlim=c(0,1), ylim=c(-45,45),
     xlab="year",ylab="thickness",axes=F,type="n")
box()
axis(2,cex=.3)
axis(1,at=unique(Arosa$x2)[c(1,9,18,27,36,45)],
       label=c("1926","1935","1944","1953","1962","1971"),cex=0.3)
polygon(c(grid2$x2,rev(grid2$x2)),
        c(arosa.ps.pred22$fit-1.96*arosa.ps.pred22$pstd,
          rev(arosa.ps.pred22$fit+1.96*arosa.ps.pred22$pstd)),
        col="lightgray",border=NA)
points(unique(Arosa$x2),m2,pch="o",cex=0.7)
lines(grid2$x2,arosa.ps.pred22$fit)
mtext("Main effect of year",cex=1.2)
dev.off()


# fit a SS ANOVA model with both month and year effects
Arosa$x1 <- (Arosa$month-0.5)/12
Arosa$x2 <- (Arosa$year-1)/45
arosa.ssanova.fit1 <- ssr(thick~I(x2-0.5), data=Arosa,
  rk=list(periodic(x1), 
          cubic(x2), 
          rk.prod(periodic(x1), kron(x2-.5)), 
          rk.prod(periodic(x1), cubic(x2))))
summary(arosa.ssanova.fit1)

# Cosine diagnostics
f1 <- predict(arosa.ssanova.fit1, terms=c(0,0,1,0,0,0))
f2 <- predict(arosa.ssanova.fit1, terms=c(0,1,0,1,0,0))
f12 <- predict(arosa.ssanova.fit1, terms=c(0,0,0,0,1,1))
fs1 <- scale(f1$fit, scale=F)
fs2 <- scale(f2$fit, scale=F)
fs12 <- scale(f12$fit, scale=F)
ys <- fs1+fs2+fs12
pi1 <- sum(fs1*ys)/sum(ys**2)
pi2 <- sum(fs2*ys)/sum(ys**2)
pi12 <- sum(fs12*ys)/sum(ys**2)
print(round(c(pi1,pi2,pi12),4))

#Euclidean norm
print(round(sqrt(c(sum(fs1**2),sum(fs2**2),sum(fs12**2))),2))

# plot interaction
grid <- seq(0,1,len=50)
arosa.main1.fit1 <- predict(arosa.ssanova.fit1,
  data.frame(x1=grid,x2=.5), 
  terms=c(0,0,1,0,0,0))
arosa.main2.fit1 <- predict(arosa.ssanova.fit1,
  data.frame(x1=.5,x2=grid), 
  terms=c(0,1,0,1,0,0))
arosa.int.fit1 <- predict(arosa.ssanova.fit1, 
  expand.grid(x1=grid,x2=grid), 
  terms=c(0,0,0,0,1,1))

ngrid <- length(grid)
d <- matrix(arosa.int.fit1$fit[1:ngrid**2],ngrid,ngrid)
lb <- d-1.96*matrix(arosa.int.fit1$pstd[1:ngrid**2],ngrid,ngrid)
ub <- d+1.96*matrix(arosa.int.fit1$pstd[1:ngrid**2],ngrid,ngrid)
z <- 0*d
z[lb>0] <- 1  # significantly larger
z[ub<0] <- -1 # significantly smaller
postscript(file="arosa.int.ps",height=3,width=3.5,pointsize=10,horizontal=F)
par(mgp=c(2,1,0), mar=c(3,3,2,1))
image(12*grid+.5, 1926+grid*45, z, col=gray(c(.8,1,.6)),
      xlab="month", ylab="year")
contour(12*grid+.5, 1926+grid*45, d, add=T, nlevels=15)
dev.off()

# drop interactions
arosa.ssanova.fit2 <- update(arosa.ssanova.fit1, 
  rk=list(periodic(x1), cubic(x2)))
summary(arosa.ssanova.fit2)

# AIC and BIC
aic.ssanova.fit1 <- nrow(Arosa)*log(sum(arosa.ssanova.fit1$resi**2))
  +2*arosa.ssanova.fit1$df
bic.ssanova.fit1 <- nrow(Arosa)*log(sum(arosa.ssanova.fit1$resi**2))
  +log(nrow(Arosa))*arosa.ssanova.fit1$df
aic.ssanova.fit2 <- nrow(Arosa)*log(sum(arosa.ssanova.fit2$resi**2))
  +2*arosa.ssanova.fit2$df
bic.ssanova.fit2 <- nrow(Arosa)*log(sum(arosa.ssanova.fit2$resi**2))
  +log(nrow(Arosa))*arosa.ssanova.fit2$df

# plot main effects
m1 <- sapply(split(Arosa$thick,Arosa$month),mean)-mean(Arosa$thick)
m2 <- sapply(split(Arosa$thick,Arosa$year),mean)-mean(Arosa$thick)
grid1 <- data.frame(x1=seq(.5/12,11.5/12,len=100), x2=.5)
grid2 <- data.frame(x1=.5, x2=seq(0,1,len=100))
arosa.ssanova.pred21 <- predict(arosa.ssanova.fit2,
  grid1,terms=c(0,0,1,0))
arosa.ssanova.pred22 <- predict(arosa.ssanova.fit2,
  grid2,terms=c(0,1,0,1))
	
postscript(file="arosa4.ps",height=2.1,width=4.4,
           horizontal=F,pointsize=6)
par(mfrow=c(1,2), mgp=c(2,1,0), mar=c(3,3,3,1))
plot(sort(unique(Arosa$x)),m1,xlim=c(0,1),ylim=c(-55,50),
     xlab="month",ylab="thickness",axes=F,type="n")
box()
axis(2,cex=.3)
axis(1,at=sort(unique(Arosa$x)),label=as.character(1:12),cex=0.3)
polygon(c(grid1$x1,rev(grid1$x1)),
  c(arosa.ssanova.pred21$fit-1.96*arosa.ssanova.pred21$pstd,
  rev(arosa.ssanova.pred21$fit+1.96*arosa.ssanova.pred21$pstd)),
  col="lightgray",border=NA)
points(sort(unique(Arosa$x)),m1,pch="o",cex=0.7)
lines(grid1$x1,arosa.ssanova.pred21$fit)
mtext("Main effect of month",cex=1.2)

plot(unique(Arosa$x2),m2,xlim=c(0,1), ylim=c(-45,45),
     xlab="year",ylab="thickness",axes=F,type="n")
box()
axis(2,cex=.3)
axis(1,at=unique(Arosa$x2)[c(1,9,18,27,36,45)],
       label=c("1926","1935","1944","1953","1962","1971"),cex=0.3)
polygon(c(grid2$x2,rev(grid2$x2)),
        c(arosa.ssanova.pred22$fit-1.96*arosa.ssanova.pred22$pstd,
          rev(arosa.ssanova.pred22$fit+1.96*arosa.ssanova.pred22$pstd)),
        col="lightgray",border=NA)
points(unique(Arosa$x2),m2,pch="o",cex=0.7)
lines(grid2$x2,arosa.ssanova.pred22$fit)
mtext("Main effect of year",cex=1.2)
dev.off()


# check partial spline model using l-spline marginal
arosa.ssanova.fit3 <- ssr(thick~sin(2*pi*x1)+cos(2*pi*x1)
 +I(x2-0.5)+I(sin(2*pi*x1)*(x2-0.5))+I(cos(2*pi*x1)*(x2-0.5)),
  rk=list(lspline(x1,type="sine1"), 
          cubic(x2), 
          rk.prod(kron(sin(2*pi*x1)),cubic(x2)),
          rk.prod(kron(cos(2*pi*x1)),cubic(x2)),
          rk.prod(lspline(x1,type="sine1"), kron(x2-.5)), 
          rk.prod(lspline(x1,type="sine1"), cubic(x2))),
 data=Arosa)

arosa.ssanova.fit4 <- ssr(thick~sin(2*pi*x1)+cos(2*pi*x1)+I(x2-0.5),
  rk=list(lspline(x1,type="sine1"), 
          cubic(x2)), 
 data=Arosa)

# put plots together
ngrid <- 50
grid1 <- seq(.5/12,11.5/12,length=ngrid)
grid2 <- seq(0,1,length=ngrid)
arosa.int.fit1 <- predict(arosa.ssanova.fit1, 
  expand.grid(x1=grid1,x2=grid2), 
  terms=c(0,0,0,0,1,1))
d <- matrix(arosa.int.fit1$fit[1:ngrid**2],ngrid,ngrid)
lb <- d-1.96*matrix(arosa.int.fit1$pstd[1:ngrid**2],ngrid,ngrid)
ub <- d+1.96*matrix(arosa.int.fit1$pstd[1:ngrid**2],ngrid,ngrid)
z <- 0*d
z[lb>0] <- 1  # significantly larger
z[ub<0] <- -1 # significantly smaller
postscript(file="arosa.int1.ps",height=2,width=4.4,pointsize=10,horizontal=F)
par(mfrow=c(1,3),mgp=c(2,1,0), mar=c(3,3,2,1))

image(12*grid1+.5, 1926+grid2*45, z, col=gray(c(.8,1,.6)),
      xlab="month", ylab="year")
contour(12*grid1+.5, 1926+grid2*45, d, add=T, 
  levels=c(-3,-2,-1,0,1,2,3), labcex=.6, lwd=.5)
mtext("(a)")

arosa.int.fit3 <- predict(arosa.ssanova.fit3, 
  expand.grid(x1=grid1,x2=grid2), 
  terms=c(0,0,0,0,1,1,0,0,1,1,1,1))
d <- matrix(arosa.int.fit3$fit[1:ngrid**2],ngrid,ngrid)
lb <- d-1.96*matrix(arosa.int.fit3$pstd[1:ngrid**2],ngrid,ngrid)
ub <- d+1.96*matrix(arosa.int.fit3$pstd[1:ngrid**2],ngrid,ngrid)
z <- 0*d
z[lb>0] <- 1  # significantly larger
z[ub<0] <- -1 # significantly smaller
image(12*grid1+.5, 1926+grid2*45, z, col=gray(c(.8,1,.6)),
      xlab="month", ylab="year")
contour(12*grid1+.5, 1926+grid2*45, d, add=T, 
  levels=c(-3,-2,-1,0,1,2,3), labcex=.6, lwd=.5)
mtext("(b)")

arosa.smain1.fit4 <- predict(arosa.ssanova.fit4, 
  expand.grid(x1=grid1,x2=0), 
  terms=c(0,0,0,0,1,0))
plot(12*grid1+.5, arosa.smain1.fit4$fit, ylim=c(-10,10),
     xlab="month",ylab="thickness",type="n")
polygon(c(12*grid1+.5,rev(12*grid1+.5)),
  c(arosa.smain1.fit4$fit-1.96*arosa.smain1.fit4$pstd,
  rev(arosa.smain1.fit4$fit+1.96*arosa.smain1.fit4$pstd)),
  col="lightgray",border=NA)
lines(12*grid1+.5,arosa.smain1.fit4$fit)
mtext("(c)")
abline(0,0,lty=2, lwd=.5)

dev.off()


# show unequal variances
v <- sapply(split(arosa.ls.fit$resi,Arosa$month),var)
a <- sort(unique(Arosa$x))
b <- lm(log(v)~sin(2*pi*a)+cos(2*pi*a))
summary(b)

# fit l-spline with unequal variances
arosa.ls.fit1 <- ssr(thick~sin(2*pi*x)+cos(2*pi*x), 
                  rk=lspline(x,type="sine1"),
                  weights=varComb(varExp(form=~sin(2*pi*x)), 
                                  varExp(form=~cos(2*pi*x))),
                  spar="m", data=Arosa)
summary(arosa.ls.fit1)
arosa.ls.pred1 <- predict(arosa.ls.fit1, newdata=data.frame(x=grid))

grid <- seq(0,1,len=50)	
postscript(file="arosa5.ps",height=2,width=4.4,
           horizontal=F,pointsize=8)
par(mfrow=c(1,2), mgp=c(2,1,0), mar=c(3,3,3,1)+0.1)
plot(a,log(v),xlim=c(0,1),ylim=c(4.5,6.7),xlab="month",ylab="log(variance)",
     axes=F,cex=.8,type="n")
points(a,log(v),pch="o",cex=.3)
box()
axis(2,cex=.2)
axis(1,at=a,label=as.character(1:12),cex=0.2)
lines(grid, log(arosa.ls.fit1$rkpk.obj$varht) 
      +2*arosa.ls.fit1$var.est[[1]]*sin(2*pi*grid) 
      +2*arosa.ls.fit1$var.est[[2]]*cos(2*pi*grid))	
lines(grid,b$coef[1]+b$coef[2]*sin(2*pi*grid)
      +b$coef[3]*cos(2*pi*grid),lty=2)		
mtext("(a)")
plot(Arosa$x,Arosa$thick,xlim=c(0,1),xlab="month",ylab="thickness",axes=F,cex=.8,type="n") 
box()
axis(2,cex=.3)
axis(1,at=a,label=as.character(1:12),cex=0.3)
polygon(c(grid,rev(grid)),
        c(arosa.ls.pred1$fit-1.96*arosa.ls.pred1$pstd,
          rev(arosa.ls.pred1$fit+1.96*arosa.ls.pred1$pstd)),
        col="lightgray",border=NA)
points(Arosa$x,Arosa$thick,pch=".",cex=0.3)
lines(grid, arosa.ls.pred1$fit)
mtext("(b)")
dev.off()


# add AR(1) correlation
Arosa$time <- Arosa$month+12*(Arosa$year-1)
arosa.ls.fit2 <- update(arosa.ls.fit1, corr=corCAR1(form=~time)) 
summary(arosa.ls.fit2)


# fit a partial spline model to the long series
Arosa$t <- (Arosa$month-0.5)/12+Arosa$year-1
arosa.ls.fit3 <- ssr(thick~t+sin(2*pi*t)+cos(2*pi*t), 
  rk=lspline(2*pi*t,type="linSinCos"), spar="m",
  corr=corExp(form=~t,nugget=T), data=Arosa)
summary(arosa.ls.fit3)

# fit a semi-parametric mixed effect model
tau <- coef(arosa.ls.fit3$cor.est, F)
D <- corMatrix(Initialize(corExp(tau[1],form=~t), data=Arosa))
Z <- chol.new(D)
arosa.ls.fit4 <- slm(thick~t+sin(2*pi*t)+cos(2*pi*t),
#  rk=cubic2(t),
  rk=lspline(2*pi*t,type="linSinCos"),
  random=list(pdIdent(~Z-1)), data=Arosa)	
summary(arosa.ls.fit4)

tm <- matrix(c(1,1,1,1,1,1,0,1,1,0,0,1,0,0,1),ncol=5,byrow=T)
grid3 <- data.frame(t=seq(0,max(Arosa$t)+0.001,len=500))
arosa.ls.fit3.p <- predict(arosa.ls.fit3, newdata=grid3, terms=tm)
arosa.ls.fit4.p <- intervals(arosa.ls.fit4, newdata=grid3, terms=tm)
newdata <- data.frame(t=c(Arosa$t,grid3$t))
RD <- corMatrix(Initialize(corExp(tau[1],form=~t), data=newdata))
R <- RD[(length(Arosa$t)+1):length(newdata$t),1:length(Arosa$t)] 
b <- as.vector(arosa.ls.fit4$lme.obj$coef$random[[2]])
u.new <- R%*%t(solve(Z))%*%b
	
postscript(file="arosa6.ps",height=6.5,width=4.4,horizontal=F,pointsize=10)	
par(mfrow=c(3,1), mgp=c(2,1,0), mar=c(4,3,3,1)+0.1)

plot(Arosa$t,Arosa$thick,xlab="time",ylab="thickness",pch="o",cex=.6)
axis(3,at=c(1926,1935,1944,1953,1962,1971)-1926+.5,
       label=c("1926","1935","1944","1953","1962","1971"),cex=0.3,lwd=0,line=-1)
axis(3,at=c(1926,1935,1944,1953,1962,1971)-1926+.5,
       label=rep("",6),tck=.02)
lines(grid3$t,arosa.ls.fit3.p$fit[,1])
mtext("(a) observations and overall fit",line=2)

plot(grid3$t,arosa.ls.fit3.p$fit[,2],xlab="time",ylab="thickness",type="l")
axis(3,at=c(1926,1935,1944,1953,1962,1971)-1926+.5,
       label=c("1926","1935","1944","1953","1962","1971"),cex=0.3,lwd=0,line=-1)
axis(3,at=c(1926,1935,1944,1953,1962,1971)-1926+.5,
       label=rep("",6),tck=.02)
mtext("(b) seasonal trend",line=2)

yr <- range(arosa.ls.fit3.p$fit[,3]-1.96*arosa.ls.fit3.p$pstd[,3],
            arosa.ls.fit3.p$fit[,3]+1.96*arosa.ls.fit3.p$pstd[,3])
plot(grid3$t,arosa.ls.fit3.p$fit[,3],xlab="time",ylab="thickness",
     type="n",ylim=yr)
axis(3,at=c(1926,1935,1944,1953,1962,1971)-1926+.5,
       label=c("1926","1935","1944","1953","1962","1971"),cex=0.3,lwd=0,line=-1)
axis(3,at=c(1926,1935,1944,1953,1962,1971)-1926+.5,
       label=rep("",6),tck=.02)
polygon(c(grid3$t,rev(grid3$t)),
        c(arosa.ls.fit3.p$fit[,3]-1.96*arosa.ls.fit3.p$pstd[,3],
          rev(arosa.ls.fit3.p$fit[,3]+1.96*arosa.ls.fit3.p$pstd[,3])),
        col="lightgray",border=NA)
lines(grid3$t,arosa.ls.fit3.p$fit[,3])
mtext("(c) long-term trend",line=2)
dev.off()

postscript(file="arosa7.ps",height=6.5,width=4.4,horizontal=F,pointsize=10)	
par(mfrow=c(4,1), mgp=c(2,1,0), mar=c(4,3,3,1)+0.1)

plot(Arosa$t,Arosa$thick,xlab="time",ylab="thickness",pch="o",cex=.6)
axis(3,at=c(1926,1935,1944,1953,1962,1971)-1926+.5,
       label=c("1926","1935","1944","1953","1962","1971"),cex=0.3,lwd=0,line=-1)
axis(3,at=c(1926,1935,1944,1953,1962,1971)-1926+.5,
       label=rep("",6),tck=.02)
lines(grid3$t,arosa.ls.fit4.p$fit[,1])
mtext("(a) observations and overall fit",line=2)

plot(grid3$t,arosa.ls.fit4.p$fit[,2],xlab="time",ylab="thickness",type="l")
axis(3,at=c(1926,1935,1944,1953,1962,1971)-1926+.5,
       label=c("1926","1935","1944","1953","1962","1971"),cex=0.3,lwd=0,line=-1)
axis(3,at=c(1926,1935,1944,1953,1962,1971)-1926+.5,
       label=rep("",6),tck=.02)
mtext("(b) seasonal trend",line=2)

yr <- range(arosa.ls.fit4.p$fit[,3]-1.96*arosa.ls.fit4.p$pstd[,3],
            arosa.ls.fit4.p$fit[,3]+1.96*arosa.ls.fit4.p$pstd[,3])
plot(grid3$t,arosa.ls.fit4.p$fit[,3],xlab="time",ylab="thickness",
     type="n",ylim=yr)
axis(3,at=c(1926,1935,1944,1953,1962,1971)-1926+.5,
       label=c("1926","1935","1944","1953","1962","1971"),cex=0.3,lwd=0,line=-1)
axis(3,at=c(1926,1935,1944,1953,1962,1971)-1926+.5,
       label=rep("",6),tck=.02)
polygon(c(grid3$t,rev(grid3$t)),
        c(arosa.ls.fit4.p$fit[,3]-1.96*arosa.ls.fit4.p$pstd[,3],
          rev(arosa.ls.fit4.p$fit[,3]+1.96*arosa.ls.fit4.p$pstd[,3])),
        col="lightgray",border=NA)
lines(grid3$t,arosa.ls.fit4.p$fit[,3])
mtext("(c) long-term trend",line=2)

plot(grid3$t,u.new,xlab="time",ylab="thickness",type="l")
axis(3,at=c(1926,1935,1944,1953,1962,1971)-1926+.5,
       label=c("1926","1935","1944","1953","1962","1971"),cex=0.3,lwd=0,line=-1)
axis(3,at=c(1926,1935,1944,1953,1962,1971)-1926+.5,
       label=rep("",6),tck=.02)
mtext("(d) local stochastic trend",line=2)
dev.off()



    