# Code for Canadian weather data

library(assist)
library(lattice)
library(fda)
attach(CanadianWeather)

# fit FLM (15.2) in Ramsay and Silverman
y <- log(apply(monthlyPrecip,2,sum))
W <- monthlyTemp
s <- apply(W,2,mean)
x <- seq(0.5,11.5,1)/12  
Sigma <- t(W)%*%periodic(x)%*%W/144
canada.fit1 <- ssr(y~s, rk=Sigma, spar="m")
summary(canada.fit1)

xgrid <- seq(0,1,len=50)
S <- periodic(xgrid,x)%*%W/12
fhat <- canada.fit1$coef$d[2]+S%*%canada.fit1$coef$c

# simple computation of bootstrap confidence intervals
nboot <- 9999
fb <- NULL
for (i in 1:nboot) {
  yb <- canada.fit1$fit+sample(canada.fit1$resi,35,replace=T)
  bfit <- ssr(yb~s, rk=Sigma, spar="m")
  fb <- cbind(fb,bfit$coef$d[2] + S%*%bfit$coef$c)
}
lb <- apply(fb,1,quantile,prob=.025)
ub <- apply(fb,1,quantile,prob=.975)

postscript(file="canada1.ps",height=2,width=3,horizontal=F,pointsize=8)
par(mgp=c(2,1,0), mar=c(3,3,0,1)+.1)
plot(xgrid,fhat,type="l",xlab="month",ylab="weight function f(x)",
     cex=.8, axes=F, xlim=c(0,1),ylim=range(lb,ub))
polygon(c(xgrid,rev(xgrid)), c(lb,rev(ub)), col=gray(0:8/8)[8], border=NA)
lines(xgrid, fhat, lwd=1.5)
abline(0,0,lty=3)
box()
axis(2,cex=.5)
axis(1,at=(1:12-.5)/12, 
 label=c("J","F","M","A","M","J","J","A","S","O","N","D"),cex=0.5)
dev.off()


# fit FLM (13.1) in Ramsay and Silverman
x1 <- rep(as.factor(region),rep(12,35))
x2 <- (rep(1:12,35)-.5)/12
y <- as.vector(monthlyTemp)

canada.fit2 <- ssr(y~1, 
               rk=list(shrink1(x1), periodic(x2),
                       rk.prod(shrink1(x1),periodic(x2))))
summary(canada.fit2)

xgrid <- seq(.5/12,11.5/12,len=50)
zone <- c("Atlantic","Pacific","Continental","Arctic")
grid <- data.frame(x1=rep(zone,rep(50,4)),x2=rep(xgrid,4))
canada.fit2.p1 <- predict(canada.fit2, newdata=grid,
                     terms=c(1,1,1,1))
# double check this is the same as 
canada.fit2.p1 <- predict(canada.fit2, newdata=grid)


trellis.device(postscript,file="canada2.ps",width=4.6,height=3.5, 
               horizontal=F) 
levels(zone) <- c("Atlantic","Pacific","Continental","Arctic")
tmp1 <- data.frame(y=y,month=x2,zone=x1)
tmp2 <- data.frame(y=canada.fit2.p1$fit,month=rep(xgrid,4),
                   zone=rep(c("Atlantic","Pacific","Continental",
                              "Arctic"),rep(length(xgrid),4)))
tmp1$rowType <- rep("data", 35*12)
tmp2$rowType <- rep("fit", nrow(tmp2))
all <- rbind(tmp1, tmp2)
all$zone <- ordered(all$zone,
  level=c("Continental","Arctic","Atlantic","Pacific"))
xyplot(y~I(12*month+.5)|zone, data=all, subscripts=T, groups=all$rowType,
       strip = function(...) strip.default(..., style = 1), 
       strip.names=c(F,F), layout=c(2,2),
       par.strip.text = list(cex = 0.75),
       scales=list(cex=.75),
       xlab=list("month",cex=.75),
       ylab=list("temperature (C)",cex=.75),
       panel=function(x,y,subscripts,groups,...){
             data <- groups[subscripts]=="data"
             fit <- groups[subscripts]=="fit"
             for (i in 1:(length(x[data]/12))) {
             panel.xyplot(x[data][(12*(i-1)+1):(12*i)], 
              y[data][(12*(i-1)+1):(12*i)], cex=0.02, type="b", lwd=.05, color="lightgray")
             }
             panel.superpose(x[fit], y[fit], 
             subscripts[fit], groups, type="l",lty=1,lwd=1.5)})
dev.off()


canada.fit2.p2 <- predict(canada.fit2,newdata=grid,
                     terms=c(0,1,0,1))
trellis.device(postscript,file="canada3.ps",width=4.6,height=3.5, 
               horizontal=F) 
tmp1 <- data.frame(y=canada.fit2.p2$fit,
                   month=rep(xgrid,4),
                   zone=rep(c("Atlantic","Pacific","Continental",
                              "Arctic"),rep(length(xgrid),4)))
tmp2 <- data.frame(y=canada.fit2.p2$fit-1.96*canada.fit2.p2$pstd,
                   month=rep(xgrid,4),
                   zone=rep(c("Atlantic","Pacific","Continental",
                              "Arctic"),rep(length(xgrid),4)))
tmp3 <- data.frame(y=canada.fit2.p2$fit+1.96*canada.fit2.p2$pstd,
                   month=rep(xgrid,4),
                   zone=rep(c("Atlantic","Pacific","Continental",
                              "Arctic"),rep(length(xgrid),4)))
tmp1$rowType <- rep("fit", nrow(tmp1))
tmp2$rowType <- rep("lb", nrow(tmp2))
tmp3$rowType <- rep("ub", nrow(tmp3))
all <- rbind(tmp1, tmp2, tmp3)
all$zone <- ordered(all$zone,
  level=c("Continental","Arctic","Atlantic","Pacific"))
xyplot(y~I(12*month+.5)|zone, data=all, subscripts=T, groups=all$rowType,
       strip = function(...) strip.default(..., style = 1), 
       strip.names=c(F,F), layout=c(2,2),
       par.strip.text = list(cex = 0.75),
       scales=list(cex=.75),
       xlab=list("month",cex=.75),
#       ylab=list(expression(alpha[g]),cex=.75),
       ylab=list("region effect",cex=.75),
       panel=function(x,y,subscripts,groups,...){
             data <- groups[subscripts]=="data"
             fit <- groups[subscripts]=="fit"
             lb <- groups[subscripts]=="lb"
             ub <- groups[subscripts]=="ub"
             panel.xyplot(x[fit],y[fit],type="n")
             panel.polygon(c(x[lb],rev(x[ub])),c(y[lb],rev(y[ub])),  
             col=gray(0:8/8)[8],border=NA)
             llines(c(0,12.5),c(0,0),lty=3)
             llines(x[fit],y[fit])})
dev.off()


# fit concurrent FLM model (14.1) in Ramsay and Silverman
x1 <- rep(as.factor(region),rep(12,35))
x2 <- (rep(1:12,35)-.5)/12
y <- log(as.vector(monthlyPrecip))
w <- canada.fit2$resi
canada.fit3 <- ssr(y~w, 
               rk=list(shrink1(x1),
                       periodic(x2),
                       rk.prod(shrink1(x1),periodic(x2)),
                       rk.prod(kron(w),periodic(x2))))
summary(canada.fit3)

xgrid <- seq(0,1,len=50)
grid <- data.frame(x1=rep(zone[1],50),x2=xgrid,w=rep(1,50))
canada.fit3.p1 <- predict(canada.fit3,newdata=grid,terms=c(0,0,0,0,0,1))
lb <- canada.fit3.p1$fit-1.96*canada.fit3.p1$pstd
ub <- canada.fit3.p1$fit+1.96*canada.fit3.p1$pstd
postscript(file="canada4.ps",height=2.2,width=3,horizontal=F,pointsize=8)
par(mgp=c(2,1,0), mar=c(3,3,1,1)+.1)
plot(xgrid,canada.fit3.p1$fit,type="l",xlab="month",ylab=expression(f[2]),
     cex=.8, axes=F, xlim=c(0,1),ylim=range(lb,ub))
polygon(c(xgrid,rev(xgrid)), c(lb,rev(ub)), col=gray(0:8/8)[8], border=NA)
lines(xgrid, canada.fit3.p1$fit, lwd=1.5)
#abline(0,0,lty=3)
box()
axis(2,cex=.5)
axis(1,at=(1:12-.5)/12, 
 label=c("J","F","M","A","M","J","J","A","S","O","N","D"),cex=0.5)
dev.off()


zone <- c("Atlantic","Pacific","Continental","Arctic")
grid <- data.frame(x1=rep(zone,rep(50,4)),x2=rep(xgrid,4),
             w=rep(0,200))
canada.fit3.p2 <- predict(canada.fit3,newdata=grid,
                     terms=c(0,0,1,0,1,0))
trellis.device(postscript,file="canada5.ps",width=4.6,height=3.5, 
               horizontal=F) 
tmp1 <- data.frame(y=canada.fit3.p2$fit,
                   month=rep(xgrid,4),
                   zone=rep(c("Atlantic","Pacific","Continental",
                              "Arctic"),rep(length(xgrid),4)))
tmp2 <- data.frame(y=canada.fit3.p2$fit-1.96*canada.fit3.p2$pstd,
                   month=rep(xgrid,4),
                   zone=rep(c("Atlantic","Pacific","Continental",
                              "Arctic"),rep(length(xgrid),4)))
tmp3 <- data.frame(y=canada.fit3.p2$fit+1.96*canada.fit3.p2$pstd,
                   month=rep(xgrid,4),
                   zone=rep(c("Atlantic","Pacific","Continental",
                              "Arctic"),rep(length(xgrid),4)))
tmp1$rowType <- rep("fit", nrow(tmp1))
tmp2$rowType <- rep("lb", nrow(tmp2))
tmp3$rowType <- rep("ub", nrow(tmp3))
all <- rbind(tmp1, tmp2, tmp3)
all$zone <- ordered(all$zone,
  level=c("Continental","Arctic","Atlantic","Pacific"))
xyplot(y~I(12*month+.5)|zone, data=all, subscripts=T, groups=all$rowType,
       strip = function(...) strip.default(..., style = 1), 
       strip.names=c(F,F), layout=c(2,2),
       par.strip.text = list(cex = 0.75),
       scales=list(cex=.75),
       xlab=list("month",cex=.75),
       ylab=list("region effect",cex=.75),
       panel=function(x,y,subscripts,groups,...){
             data <- groups[subscripts]=="data"
             fit <- groups[subscripts]=="fit"
             lb <- groups[subscripts]=="lb"
             ub <- groups[subscripts]=="ub"
             panel.xyplot(x[fit],y[fit],type="n")
             panel.polygon(c(x[lb],rev(x[ub])),c(y[lb],rev(y[ub])),  
             col=gray(0:8/8)[8],border=NA)
             llines(c(0,12.5),c(0,0),lty=3)
             llines(x[fit],y[fit])})
dev.off()


# fit FLM model (16.1) in Ramsay and Silverman
W <- monthlyTemp
z <- apply(W,2,mean)
s <- rep(z,rep(12,35))
x <- seq(0.5,11.5,1)/12
y <- log(as.vector(monthlyPrecip))
Q1 <- kronecker(matrix(1,35,35),periodic(x))
Q2 <- kronecker(t(W)%*%periodic(x)%*%W,matrix(1,12,12))/144
Q3 <- kronecker(z%*%t(z),periodic(x))
Q4 <- rk.prod(Q1,Q2)
canada.fit4 <- ssr(y~s, rk=list(Q1,Q2,Q3,Q4))
summary(canada.fit4)

# compute fits 
ngrid <- 40
xgrid <- seq(0,1,len=ngrid)
S1 <- kronecker(t(rep(1,35)),periodic(xgrid,x))
S2 <- kronecker(periodic(xgrid,x)%*%W,t(rep(1,12)))/12
S3 <- kronecker(t(z),periodic(xgrid,x))
S4 <- NULL
for (k in 1:ngrid) {
  for (l in 1:ngrid) S4 <- rbind(S4,S1[l,]*S2[k,])}

the <- 10^canada.fit4$rkpk.obj$theta
f1 <- canada.fit4$coef$d[1] + the[1]*S1%*%canada.fit4$coef$c
mu2 <- canada.fit4$coef$d[2]
f21 <- the[2]*S2%*%canada.fit4$coef$c
f22 <- the[3]*S3%*%canada.fit4$coef$c
f212 <- the[4]*S4%*%canada.fit4$coef$c
f2 <- mu2+rep(f21,rep(ngrid,ngrid))+rep(f22,ngrid)+f212

postscript(file="canada6.ps",height=2.2*1.5,width=4.45*1.5,horizontal=F,
  pointsize=8)
trellis.par.set("axis.line", list(col="black"))
p1 <- xyplot(f1~xgrid,xlab=expression(x[2]),ylab=expression(f[1]),
 type="l",ylim=c(1.6,3), scale=list(tck=c(1,0),cex=.6))
print(p1, split=c(1,1,2,1), position=c(.05,.05,.95,.95), more=T)

ind <- seq(1,ngrid,by=2)
trellis.par.set("axis.line", list(col="transparent"))
a <- expand.grid(xgrid[ind],xgrid[ind])
tmp <- as.vector(t(matrix(f2,ngrid,ngrid)[ind,ind]))
p2 <- wireframe(tmp~a[[1]]*a[[2]], 
 screen=list(z=25,x=-75), xlab="u", ylab=expression(paste(x[2])),
 zlab=expression(paste(f[2])), 
 scales=list(arrows=F, cex=.6, col="black", tck=1))
print(p2, split = c(2,1,2,1))

dev.off()


