# Code for Carbon dioxide data

library(assist)
library(lattice)
data(CO2)

# fit the NLME model
co2.nlme <- nlme(uptake~exp(a1)*(1-exp(-exp(a2)*(conc-a3))), 
                 fixed=list(a1+a2~Type*Treatment,a3~1), 
                 random=a1~1, groups=~Plant, 
                 start=c(log(30),0,0,0,log(0.01),0,0,0,50),
                 data=CO2)
summary(co2.nlme)

# fit the SNM model
M <- model.matrix(~Type*Treatment, data=CO2)[,-1]
co2.snm <- snm(uptake~exp(a1)*f(exp(a2)*(conc-a3)),
                func=f(u)~list(~I(1-exp(-u))-1,lspline(u, type="exp")),
                fixed=list(a1~M-1,a3~1,a2~Type*Treatment),
                random=list(a1~1), group=~Plant, verbose=T,
                start=co2.nlme$coe$fixed[c(2:4,9,5:8)], data=CO2)
summary(co2.snm)

# plot
co2.grid <- data.frame(Plant=rep(unique(CO2$Plant),rep(50,12)),
                       Type=rep(unique(CO2$Type),rep(300,2)),
                       Treatment=rep(rep(unique(CO2$Treatment),2),rep(150,4)),
                       conc=rep(seq(95,1000,len=50),12))        
M <- model.matrix(~Type*Treatment, data=co2.grid)[,-1]
co2.snm.p <- predict(co2.snm,newdata=co2.grid)
M <- model.matrix(~Type*Treatment, data=CO2)[,-1]

trellis.device(postscript, file="co21.ps", width=4.4, height=5.6, 
               horizontal=F, theme=list(fontsize=list(text=8, points=6)))
subj <- c("Qn1","Qn2","Qn3","Qc1","Qc2","Qc3","Mn1","Mn2","Mn3","Mc1","Mc2","Mc3")
tmp1 <- data.frame(uptake=CO2$uptake, conc=CO2$conc, Plant=rep(subj,rep(7,12)))
tmp2 <- data.frame(uptake=co2.nlme$fitted[,2], conc=CO2$conc, Plant=rep(subj,rep(7,12)))
tmp3 <- data.frame(uptake=co2.snm.p, conc=co2.grid$conc, Plant=rep(subj,rep(50,12)))
tmp1$rowType <- rep("original", 84)
tmp2$rowType <- rep("nlmeFit", 84)
tmp3$rowType <- rep("snmFit", nrow(tmp3))
co2Fit <- rbind(tmp1, tmp2, tmp3)
tmp1$Plant<- ordered(tmp1$Plant,level=subj)
xyplot(uptake~conc|Plant, data=co2Fit, subscripts=T, groups=co2Fit$rowType,
       strip = function(...) strip.default(..., style = 1), 
       strip.names=c(F,F), layout=c(3,4),
       xlab="ambient carbon dioxide concentration(uL/L)",
       ylab=expression(paste("C",O[2]," uptake rate (umol/m"^{2}," s)")),
       panel=function(x,y,subscripts,groups,...){
             orig <- groups[subscripts]=="original"
             fit2 <- groups[subscripts]=="snmFit"
             fit1 <- groups[subscripts]=="nlmeFit"
             panel.xyplot(x[orig], y[orig], cex=0.5)
             panel.superpose(x[fit2], y[fit2], subscripts[fit2], groups, type="l")
             panel.superpose(x[fit1], y[fit1], subscripts[fit1], groups, type="l")})
graphics.off()

# construct Bayesian confidence intervals for the overall fit and its projections
co2.grid2 <- data.frame(u=seq(0.3, 11, len=50))
co2.ci <- intervals(co2.snm, newdata=co2.grid2,
                    terms=matrix(c(1,1,1,0,0,1), ncol=2,byrow=T))
trellis.device(postscript, file="co22.ps", width=4.4, height=2.2, 
               horizontal=F, theme=list(fontsize=list(text=8, points=4)))
plot(co2.ci, type.name=c("overall","parametric","smooth"))
graphics.off()


