# Code for hormone data

library(assist)
library(lattice)
data(horm.cort)

# 1. (a) fit a fixed effect SIM model	
nor <- horm.cort[horm.cort$type=="normal",] 
M <- model.matrix(~as.factor(ID), data=nor)
nor.snr.fit1 <- snr(conc~b1+exp(b2)*f(time-alogit(b3)), 
 func=f(u)~list(~sin(2*pi*u)+cos(2*pi*u)-1,lspline(u,type="sine0")),
 params=list(b1~M-1, b2+b3~M[,-1]-1),
 start=list(params=c(mean(nor$conc),rep(0,24))), 
 data=nor, spar="m", 
 control=list(prec.out=0.001,converg="PRSS"))
summary(nor.snr.fit1)

# (b) prediction and plot
nor.grid <- data.frame(ID=rep(unique(nor$ID),rep(50,9)),
                       time=rep(seq(0,1,len=50),9))
M <- model.matrix(~as.factor(ID), data=nor.grid)
nor.snr.p <- predict(nor.snr.fit1,newdata=nor.grid)
tmp1 <- data.frame(conc=nor$conc, time=nor$time, ID=as.character(nor$ID))
tmp2 <- data.frame(conc=nor.snr.p, time=nor.grid$time, ID=as.character(nor.grid$ID))
tmp1$rowType <- rep("original", nrow(tmp1))
tmp2$rowType <- rep("snrFit", nrow(tmp2))
cortfit <- rbind(tmp1, tmp2)
cortfit$ID <- factor(cortfit$ID, levels=as.character(c(8007:8009,8004:8006,8001:8003)))
trellis.device(postscript, file="hormone1.ps", width=4.4, height=5.6, horizontal=F)
xyplot(conc~time|ID, data=cortfit, subscripts=T, groups=cortfit$rowType,
       strip = function(...) strip.default(..., style = 1), 
       strip.names=c(F,F), layout=c(3,3),
       ylab="cortisol concentration on log scale",
       xlab="time",
       panel=function(x,y,subscripts,groups,...){
             orig <- groups[subscripts]=="original"
             fit1 <- groups[subscripts]=="snrFit"
             panel.xyplot(x[orig], y[orig], cex=0.5)
             panel.superpose(x[fit1], y[fit1], subscripts[fit1], groups, type="l", lty=1)})
dev.off()
	
grid <- seq(0,1,len=100)
nor.snr.p.f <- intervals(nor.snr.fit1,newdata=data.frame(u=grid),
    terms=list(f=matrix(c(1,1,1,0,0,1),nrow=2,byrow=T)))
postscript(file="hormone2.ps",height=2,width=4.4,horizontal=F,pointsize=10)
par(mfrow=c(1,2),mgp=c(2,1,0), mar=c(3,3,1,1)+.1)

plot(grid, nor.snr.p.f$f$fit[,1],xlab="x",ylab="f",type="n",ylim=c(-1.2,1))
mtext("(a)")
polygon(c(grid,rev(grid)),
        c(nor.snr.p.f$f$fit[,1]-2*nor.snr.p.f$f$pstd[,1],
          rev(nor.snr.p.f$f$fit[,1]+2*nor.snr.p.f$f$pstd[,1])),
          col=gray(0:8/8)[8], border=NA)
lines(grid, nor.snr.p.f$f$fit[,1])

plot(grid, nor.snr.p.f$f$fit[,2],xlab="x",type="n",ylim=c(-1.2,1),
 ylab=expression(paste(P[1],f)))
mtext("(b)")
polygon(c(grid,rev(grid)),
        c(nor.snr.p.f$f$fit[,2]-2*nor.snr.p.f$f$pstd[,2],
          rev(nor.snr.p.f$f$fit[,2]+2*nor.snr.p.f$f$pstd[,2])),
        col=gray(0:8/8)[8], border=NA)
lines(grid, nor.snr.p.f$f$fit[,2])
dev.off()


# (c) SIM with AR(1) error
M <- model.matrix(~as.factor(ID), data=nor)
nor.snr.fit2 <- update(nor.snr.fit1, cor=corAR1(form=~1|ID))
summary(nor.snr.fit2)
	
# 2. (a) fit a mixed effect SIM
nor.snm.fit <- snm(conc~b1+exp(b2)*f(time-alogit(b3)), func=f(u)~list(periodic(u)),
                 data=nor, fixed=list(b1~1), random=list(b1+b2+b3~1), 
                 start=c(mean(nor$conc)), groups=~ID, spar="m")
summary(nor.snm.fit)

# (b) check AR(1) error
nor.snm.fit2 <- update(nor.snm.fit, cor=corAR1(form=~1|ID))
summary(nor.snm.fit2)

# 3. fit mixed SIM for depressed subjects
dep <- horm.cort[horm.cort$type=="depression",] 
dep.snm.fit <- snm(conc~b1+exp(b2)*f(time-alogit(b3)), func=f(u)~list(periodic(u)),
                 data=dep, fixed=list(b1~1), random=list(b1+b2+b3~1), 
                 start=c(mean(dep$conc)), groups=~ID, spar="m")
summary(dep.snm.fit)

# 4. fit mixed SIM for cushion subjects
cush <- horm.cort[horm.cort$type=="cushing",] 
cush.snm.fit <- snm(conc~b1+exp(b2)*f(time-alogit(b3)), func=f(u)~list(periodic(u)),
                    data=cush, fixed=list(b1~1), random=list(b1+b2+b3~1), 
                    start=c(mean(cush$conc)), groups=~ID, spar="m",verbose=T)
summary(cush.snm.fit)
cush.lme.fit <- lme(conc~1, data=cush, random=~1|ID)


# 5. fit normal and depression groups together
nordep <- horm.cort[horm.cort$type!="cushing",] 
nordep$type <- as.factor(as.vector(nordep$type))
nordep.fit1 <- snm(conc~b1+exp(b2)*f(type,time-alogit(b3)), 
                  func=f(g,u)~list(list(periodic(u),
                  rk.prod(shrink1(g),periodic(u)))),
                  data=nordep, fixed=list(b1~type),
                  random=list(b1+b2+b3~1), groups=~ID, 
                  weights=varIdent(form=~1|type), 
                  spar="m", start=c(1.8,-.2), verbose=T)
summary(nordep.fit1)

u <- seq(0,1,len=50)
nordep.inter <- intervals(nordep.fit1, terms=c(0,1),
 newdata=data.frame(g=rep(c("normal","depression"),c(50,50)),u=rep(u,2)))
range(nordep.inter$fit)
range(nordep.inter$pstd)
	
nordep.fit2 <- snm(conc~b1+exp(b2+d1*I(type=="normal"))*f(time-alogit(b3+d2*I(type=="normal"))), 
                   func=f(u)~list(periodic(u)),
                   data=nordep, fixed=list(b1~type,d1+d2~1), 
                   random=list(b1+b2+b3~1), groups=~ID,
                   weights=varIdent(form=~1|type), 
                   spar="m", start=c(1.9,-0.3,0,0))
summary(nordep.fit2)

# d2 not significant
nordep.fit3 <- snm(conc~b1+exp(b2+d1*I(type=="normal"))*f(time-alogit(b3)), 
                   func=f(u)~list(periodic(u)),
                   data=nordep, fixed=list(b1~type,d1~1), 
                   random=list(b1+b2+b3~1), groups=~ID, 
                   weights=varIdent(form=~1|type), 
                   spar="m", start=c(1.9,-0.3,0))
summary(nordep.fit3)

u <- seq(0,1,len=50)
nordep.grid <- data.frame(time=rep(u,20),
     type=rep(c("normal","depression"),c(450,550)),
     ID=rep(unique(nordep$ID),rep(50,20)))
nordep.p <- predict(nordep.fit3,newdata=nordep.grid)


# 6. plots of observations and fits
# prediction and plot for normal group
nor.grid <- data.frame(ID=rep(unique(nor$ID),rep(50,9)), time=rep(seq(0,1,len=50),9))
nor.snm.p <- predict(nor.snm.fit,newdata=nor.grid)
tmp1 <- data.frame(conc=nor$conc, time=nor$time, ID=as.character(nor$ID))
tmp2 <- data.frame(conc=nor.snm.p, time=nor.grid$time, ID=as.character(nor.grid$ID))
tmp3 <- data.frame(conc=nordep.p, time=nordep.grid$time, ID=as.character(nordep.grid$ID))
tmp1$rowType <- rep("original", nrow(tmp1))
tmp2$rowType <- rep("snmFit", nrow(tmp2))
tmp3$rowType <- rep("jointFit", nrow(tmp3))
cortfit <- rbind(tmp1, tmp2, tmp3)
cortfit$ID <- factor(cortfit$ID, levels=as.character(c(8007:8009,8004:8006,8001:8003)))
trellis.device(postscript, file="hormone3.ps", width=4.4, height=5.6, horizontal=F)
xyplot(conc~time|ID, data=cortfit, subscripts=T, groups=cortfit$rowType,
       strip = function(...) strip.default(..., style = 1), 
       strip.names=c(F,F), layout=c(3,3),
       ylab="cortisol concentration on log scale",
       xlab="time",
       panel=function(x,y,subscripts,groups,...){
             orig <- groups[subscripts]=="original"
             fit1 <- groups[subscripts]=="snmFit"
             fit2 <- groups[subscripts]=="jointFit"
             panel.xyplot(x[orig], y[orig], cex=0.5)
             panel.superpose(x[fit1], y[fit1], subscripts[fit1], groups, type="l", lty=1)
             panel.superpose(x[fit2], y[fit2], subscripts[fit2], groups, type="l", lty=2)})
dev.off()

# prediction and plot for depression group
dep.grid <- data.frame(ID=rep(unique(dep$ID),rep(50,11)), time=rep(seq(0,1,len=50),11))
dep.p <- predict(dep.snm.fit,newdata=dep.grid)
tmp1 <- data.frame(conc=dep$conc, time=dep$time, ID=as.character(dep$ID))
tmp2 <- data.frame(conc=dep.p, time=dep.grid$time, ID=as.character(dep.grid$ID))
tmp3 <- data.frame(conc=nordep.p, time=nordep.grid$time, ID=as.character(nordep.grid$ID))
tmp1$rowType <- rep("original", nrow(tmp1))
tmp2$rowType <- rep("snmFit", nrow(tmp2))
tmp3$rowType <- rep("jointFit", nrow(tmp3))
cortfit <- rbind(tmp1, tmp2, tmp3)
cortfit$ID <- factor(cortfit$ID, levels=as.character(c(122:124,117:119,113,115,116,111,112)))
trellis.device(postscript, file="hormone4.ps", width=4.4, height=5.6, horizontal=F)
xyplot(conc~time|ID, data=cortfit, subscripts=T, groups=cortfit$rowType,
       strip = function(...) strip.default(..., style = 1), 
       strip.names=c(F,F), layout=c(3,4),
       ylab="cortisol concentration on log scale",
       xlab="time",
       panel=function(x,y,subscripts,groups,...){
             orig <- groups[subscripts]=="original"
             fit1 <- groups[subscripts]=="snmFit"
             fit2 <- groups[subscripts]=="jointFit"
             panel.xyplot(x[orig], y[orig], cex=0.5)
             panel.superpose(x[fit1], y[fit1], subscripts[fit1], groups, type="l", lty=1)
             panel.superpose(x[fit2], y[fit2], subscripts[fit2], groups, type="l", lty=2)})
dev.off()
	
# prediction and plot for Cushing group
cush.grid <- data.frame(ID=rep(unique(cush$ID),rep(50,16)), time=rep(seq(0,1,len=50),16))
#cush.p <- predict(cush.snm.fit,newdata=cush.grid)
cush.p <- rep(ranef(cush.lme.fit)[,1], rep(50,16))+fixef(cush.lme.fit)
tmp1 <- data.frame(conc=cush$conc, time=cush$time, ID=as.character(cush$ID))
tmp2 <- data.frame(conc=cush.p, time=cush.grid$time, ID=as.character(cush.grid$ID))
tmp1$rowType <- rep("original", nrow(tmp1))
tmp2$rowType <- rep("snmFit", nrow(tmp2))
cortfit <- rbind(tmp1, tmp2)
cortfit$ID <- factor(cortfit$ID,levels=as.character(unique(cush$ID)[c(13:16,9:12,5:8,1:4)]))
trellis.device(postscript, file="hormone5.ps", width=4.4, height=5.6, horizontal=F)
xyplot(conc~time|ID, data=cortfit, subscripts=T, groups=cortfit$rowType,
       strip = function(...) strip.default(..., style = 1), 
       strip.names=c(F,F), layout=c(4,4),
       ylab="cortisol concentration on log scale",
       xlab="time",
       panel=function(x,y,subscripts,groups,...){
             orig <- groups[subscripts]=="original"
             fit1 <- groups[subscripts]=="snmFit"
             panel.xyplot(x[orig], y[orig], cex=0.5)
             panel.superpose(x[fit1], y[fit1], subscripts[fit1], groups, type="l", lty=1)})
dev.off()

# 7. compute estimate of common curves and CI's
u <- seq(0,1,len=50)
ci.grid <- data.frame(time=u)
nor.ci <- intervals(nor.snm.fit, newdata=ci.grid)
dep.ci <- intervals(dep.snm.fit, newdata=ci.grid)
cush.ci <- intervals(cush.snm.fit, newdata=ci.grid)
nordep.ci <- intervals(nordep.fit3, newdata=ci.grid)

postscript(file="hormone6.ps",height=1.8,width=4.4,horizontal=F,pointsize=8)
par(mfrow=c(1,4), mgp=c(2,1,0), mar=c(3,2,0,1)+.1)	
plot(u, nor.ci$fit, ylim=c(-2,2), xlab="time",ylab="f",type="n")
polygon(c(u,rev(u)),
        c(nor.ci$fit-1.96*nor.ci$pstd,rev(nor.ci$fit+1.96*nor.ci$pstd)),
        col=gray(0:8/8)[8], border=NA)
lines(u, nor.ci$fit)

plot(u,dep.ci$fit[,1],ylim=c(-2,2), xlab="time",ylab="f",type="n")
polygon(c(u,rev(u)),
        c(dep.ci$fit-1.96*dep.ci$pstd,rev(dep.ci$fit+1.96*dep.ci$pstd)),
        col=gray(0:8/8)[8], border=NA)
lines(u, dep.ci$fit)
plot(u,cush.ci$fit,ylim=c(-2,2), xlab="time",ylab="f",type="n")
polygon(c(u,rev(u)),
        c(cush.ci$fit-1.96*cush.ci$pstd,rev(cush.ci$fit+1.96*cush.ci$pstd)),
        col=gray(0:8/8)[8], border=NA)
lines(u, cush.ci$fit)
plot(u, nordep.ci$fit, ylim=c(-2,2), xlab="time",ylab="f",type="n")
polygon(c(u,rev(u)),
        c(nordep.ci$fit-1.96*nordep.ci$pstd,rev(nordep.ci$fit+1.96*nordep.ci$pstd)),
        col=gray(0:8/8)[8], border=NA)
lines(u, nordep.ci$fit)
graphics.off()

nor.mean <- nordep.fit3$coef$fixed[1]+nordep.fit3$coef$fixed[2]+
            nordep.fit3$coef$random$ID[12:20,1]
dep.mean <- nordep.fit3$coef$fixed[1]+nordep.fit3$coef$random$ID[1:11,1]
nor.amp <- exp(nordep.fit3$coef$fixed[3]+nordep.fit3$coef$random$ID[12:20,2])
dep.amp <- exp(nordep.fit3$coef$random$ID[1:11,2])
u <- c(nor.mean, dep.mean)
v <- c(nor.amp, dep.amp)
s <- c(rep("n",9),rep("d",11))
library(rpart)
prune(rpart(s~u+v), cp=.1)

postscript(file="hormone7.ps",height=2,width=3.2,horizontal=F,pointsize=8)
par(mgp=c(2,1,0), mar=c(3,3,1,1)+.1)	
plot(u, v, type="n", xlab="24-hour mean", ylab="amplitude", ylim=c(.5,1.5))
text(u, v, s, cex=.6)
lines(c(1.64,1.64), c(.5,1.5), lty=2)
dev.off()

