# Simulation code for spline smoothing with heteroscedastic and/or correlated errors

library(assist)
library(lattice)

# unequal variance
set.seed(3487)
nsimu <- 100
n <- 100
s <- .05
alpha <- 4
x <- 1:n/n
f <- sin(4*pi*x**2)
v <- exp(alpha*abs(f))
S <- cubic(x)

fv <- fm <- fwv <- fwm <- NULL
ci <- ciw <- array(NA,c(n,nsimu,2))
for (i in 1:nsimu) {
  y <- f+rnorm(n,sd=s*sqrt(v))
  fv <- cbind(fv,ssr(y~x,rk=S,spar="v")$fit)
  fit <- ssr(y~x,rk=S,spar="m")
  fm <- cbind(fm,fit$fit)
  pred <- predict(fit)
  ci[,i,1] <- pred$fit-1.96*pred$pstd
  ci[,i,2] <- pred$fit+1.96*pred$pstd
  fwv <- cbind(fwv,ssr(y~x,rk=S,spar="v",weights=v)$fit)
  fit <- ssr(y~x,rk=S,spar="m",weights=v)
  fwm <- cbind(fwm,fit$fit)
  pred <- predict(fit)
  ciw[,i,1] <- pred$fit-1.96*pred$pstd
  ciw[,i,2] <- pred$fit+1.96*pred$pstd
}
msev <- apply((fv-f)**2/v,2,mean)
msem <- apply((fm-f)**2/v,2,mean)
msewv <- apply((fwv-f)**2/v,2,mean)
msewm <- apply((fwm-f)**2/v,2,mean)
pcov <- apply(ci[,,1]<=f & ci[,,2]>=f,1,mean)
pcovw <- apply(ciw[,,1]<=f & ciw[,,2]>=f,1,mean)
acp <- apply(ci[,,1]<=f & ci[,,2]>=f,2,mean)
acpw <- apply(ciw[,,1]<=f & ciw[,,2]>=f,2,mean)

postscript(file="uneqvar.ps",height=3.5,width=4.4,horizontal=F,pointsize=8)
par(mfrow=c(2,2), mgp=c(2,1,0), mar=c(3,3,2,1)+0.1)

bp <- boxplot(split(c(msev,msewv,msem,msewm),
  rep(1:4,rep(nsimu,4))), names=c("GCV","GCVW","GML","GMLW"),
  ylab="WMSE")
xi <- .3+seq(bp$n)
points(seq(bp$n),c(mean(msev),mean(msewv),mean(msem),mean(msewm)),  
  pch="+",cex=1.5)
mtext("(a)")

bp <- boxplot(split(c(acp,acpw),
  rep(1:2,rep(nsimu,2))), names=c("unweighted","weighted"),
  ylab="average coverage")
xi <- .3+seq(bp$n)
points(seq(bp$n),c(mean(acp),mean(acpw)),  pch="+",cex=1.5)
abline(.95,0,lty=3)  
mtext("(b)")

plot(x,pcov,ylim=c(0,1),ylab="pointwise coverage",type="l")
lines(x,v/(5*max(v)),lty=2)
abline(.95,0,lty=3)
mtext("(c)")

plot(x,pcovw,ylim=c(0,1),ylab="pointwise coverage",type="l")
lines(x,v/(5*max(v)),lty=2)
abline(.95,0,lty=3)
mtext("(d)")

dev.off()


# correlation, trellis plot need to be generated manually
set.seed(8437)
n <- 100
s <- .2
x <- 1:n/n
f <- sin(4*pi*x**2)
S <- cubic(x)
y <- f+s*arima.sim(n,model=list(ar=0.6))
spar <- c("UBR","GCV","GML")
fitv <- ssr(y~x, rk=S, spar="v")$fit
fitm <- ssr(y~x, rk=S, spar="m")$fit
fitu <- ssr(y~x, rk=S, spar="u", varht=s**2)$fit

tmp1 <- data.frame(x=rep(x,3),y=rep(y,3),ind=rep(spar,rep(n,3)))
tmp2 <- data.frame(x=rep(x,3),y=rep(f,3),ind=rep(spar,rep(n,3)))
tmp3 <- data.frame(x=rep(x,3),y=c(fitu,fitv,fitm),ind=rep(spar,rep(n,3)))
tmp1$rowType <- rep("observation", 3*n)
tmp2$rowType <- rep("true", 3*n)
tmp3$rowType <- rep("fit", 3*n)
all <- rbind(tmp1, tmp2, tmp3)
all$ind <- factor(all$ind, levels=spar)
trellis.device(postscript, file="corr1.ps", width=5.72, height=2.5, horizontal=F,pointsize=5) 
xyplot(y~x|ind, data=all, subscripts=T, groups=all$rowType,
       strip=function(...) strip.default(..., style = 1), 
       strip.names=c(F,F), layout=c(3,1), xlab="x", ylab="y",
       par.strip.text=list(cex=0.75),
       scales=list(cex=.6,tck=.6),
       panel=function(x,y,subscripts,groups,...){
             obs <- groups[subscripts]=="observation"
             true <- groups[subscripts]=="true"
             fit <- groups[subscripts]=="fit"
             panel.xyplot(x[obs], y[obs], pch="o",cex=.5)
             panel.superpose(x[true], y[true], subscripts[true], groups, type="l", lty=2)
             panel.superpose(x[fit], y[fit], subscripts[fit], groups, type="l")})
dev.off()

postscript(file="corr2.ps",height=4,width=4.4,horizontal=F,pointsize=8)
par(mfrow=c(2,2), mgp=c(2,1,0), mar=c(3,3,3,1)+0.1)
# independent data, but residuals are correlated
set.seed(4598)
n <- 100
x <- 1:n/n
y <- sin(4*pi*x**2)+.2*rnorm(n)
plot(x,y,pch="o",cex=.6)
mtext("(a)")
#legend(0,-0.6,legend="lambda=1.E-3",cex=1,bty="n")
fit <- ssr(y~x,cubic(x), limnla=-3.5)
lines(x,sin(4*pi*x**2),lty=2)
lines(x,fit$fit)
acf(residuals(fit),main="")
mtext("(b)")

# correlated data, but residuals are independent
set.seed(394823)
y <- .2*arima.sim(n,model=list(ar=0.6))
plot(x,y,pch="o",cex=.6)
mtext("(c)", cex=1.2)
abline(0,0,lty=2)
fit <- ssr(y~x,cubic(x))
lines(x,fit$fit)
acf(residuals(fit),main="")
mtext("(d)")
dev.off()


