# Code for superconductivity data
# Superconductivity Magnetization Modeling
# http://www.itl.nist.gov/div898/strd/nls/data/LINKS/DATA/Bennett5.dat
# more information in BENNETT5.DAT downloaded from
#
#http://www.itl.nist.gov/div898/software/dataplot/datasets.htm#Regression/1-Variable%20and%20Non-Linear

library(assist)
library(NISTnls)
a <- Bennett5
x <- ident(a$x)
y <- a$y

# fit a cubic spline model
super.cub <- ssr(y~x, cubic(x), spar="m")
anova(super.cub, simu.size=1000)

# fit a nonlinear regression model
# initial values are transformed from those given in the webpage
b10 <- -1500*(max(a$x)-min(a$x))**(-1/.85)
b20 <- (45+min(a$x))/(max(a$x)-min(a$x))
super.nls <- nls(y~b1*(b2+x)**(-1/b3), 
  start=list(b1=b10,b2=b20,b3=.85))

# fit a snr model
bh <- coef(super.nls)
super.snr <- snr(y~b1*(b2+x)**(-1/b3)+f(x),
 params=list(b1+b2+b3~1), 
 func=f(u)~list(~I(u-.5), cubic(u)),
 start=list(params=bh), spar="m")

# fit an L-spline
ip.rk <- function(x,b2,b3) ((b2+x)%o%(b2+x))**(-1/b3)
super.l <- ssr(y~I((bh[2]+x)**(-1/bh[3]))-1,
  rk=ip.rk(x,bh[2],bh[3]), spar="m")
anova(super.l, simu.size=1000)

postscript(file="super1.ps",height=2.6,width=4,horizontal=F,pointsize=8)
par(mgp=c(2,1,0), mar=c(3,3,0,1)+.1)
plot(a$x, y, xlab="logarithm of time (min)", 
 ylab=expression(paste("magnetization (a",m^2,"/kg)")),type="n")
points(a$x, y, pch="o", cex=.6)
lines(a$x, super.cub$fit, lty=2)
lines(a$x, super.snr$fit, lty=3)
lines(a$x, super.l$fit, lty=4)
lines(a$x,fitted(super.nls),lty=1)
legend(7.5,-32,legend=c("NLR","cubic spline","NPS","L-spline"),lty=1:4,
 cex=.8, box.lwd=0)
dev.off()

grid <- seq(0,1,len=100)
xgrid <- min(a$x)+grid*diff(range(a$x))
super.cub.pred <- predict(super.cub, terms=c(0,0,1), 
 newdata=data.frame(x=grid))
super.snr.pred <- intervals(super.snr, newdata=data.frame(u=grid)) 
super.l.pred <- predict(super.l,terms=c(0,1))

postscript(file="super2.ps",height=1.8,width=4.4,horizontal=F,pointsize=10)
par(mfrow=c(1,3),mgp=c(2,1,0), mar=c(3,3,1,1)+.1)

yr <- range(super.cub.pred$fit-1.96*super.cub.pred$pstd,
            super.cub.pred$fit+1.96*super.cub.pred$pstd)
plot(xgrid, super.cub.pred$fit, xlab="logarithm of time (min)", 
 ylab=expression(f[1]), ylim=yr, type="n")
polygon(c(xgrid,rev(xgrid)),
        c(super.cub.pred$fit-1.96*super.cub.pred$pstd,
          rev(super.cub.pred$fit+1.96*super.cub.pred$pstd)),
          col=gray(0:8/8)[8], border=NA)
lines(xgrid, super.cub.pred$fit)
abline(0,0,lty=2)
mtext("(a)")

plot(xgrid, super.snr.pred$f$fit, xlab="logarithm of time (min)", 
 ylab=expression(f[2]), ylim=yr, type="n")
polygon(c(xgrid,rev(xgrid)),
        c(super.snr.pred$f$fit-1.96*super.snr.pred$f$pstd,
          rev(super.snr.pred$f$fit+1.96*super.snr.pred$f$pstd)),
          col=gray(0:8/8)[8], border=NA)
lines(xgrid, super.snr.pred$f$fit)
abline(0,0,lty=2)
mtext("(b)")

plot(a$x, super.l.pred$fit, xlab="logarithm of time (min)", 
 ylab=expression(f[3]), ylim=yr, type="n")
polygon(c(a$x,rev(a$x)),
        c(super.l.pred$fit-1.96*super.l.pred$pstd,
          rev(super.l.pred$fit+1.96*super.l.pred$pstd)),
          col=gray(0:8/8)[8], border=NA)
lines(a$x, super.l.pred$fit)
abline(0,0,lty=2)
mtext("(c)")

dev.off()

