# Code for Texas weather data

library(assist)
library(maps)
library(akima)
library(lattice)

data(TXtemp)
staname <- c("ALBANY","ALICE","ALPINE","BALLINGER 2NW","BALMORHEA",
             "BEEVILLE 5NE","BLANCO","BOERNE","BOYS RANCH","BRENHAM",
             "BROWNWOOD","CATARINA","CLARKSVILLE 2NE","CORPUS CHRISTI WSO AP",
             "CORSICANA","CROSBYTON","DANEVANG 1W","DUBLIN","EAGLE PASS",
             "EL PASO WSO AP","ENCINAL","FALFURRIAS","FLATONIA",
             "FORT STOCKTON","GAINESVILLE 5ENE","GREENVILLE","HALLETTSVILLE 2N",
             "HASKELL","LAMPASAS","LIBERTY","LLANO","LULING","MARSHALL",
             "MCCAMEY","MEXIA","MIAMI","MULESHOE 1","NEW BRAUNFELS",
             "PECOS","PLAINVIEW","QUANAH 5SE","RIO GRANDE CITY 3W",
             "SAN ANTONIO WSFO","SEMINOLE","SNYDER","STRATFORD",
             "TEMPLE","WEATHERFORD")

# select a subset
TXtemp1 <- TXtemp[TXtemp$year>1985,]
y <- gapply(TXtemp1, which=5, FUN=function(x) mean(x[x!=-99.99]),
            group=TXtemp1$stacod*TXtemp1$month)
tx.dat <- data.frame(y=as.vector(t(matrix(y,48,12,byrow=F))))
tx.dat$x2 <- rep((1:12-0.5)/12, 48)
lat <- TXtemp$lat[seq(1, nrow(TXtemp),by=360)]
long <- TXtemp$long[seq(1, nrow(TXtemp),by=360)]
tx.dat$lat <- rep(lat, rep(12,48))
tx.dat$long <- rep(long, rep(12,48))
tx.dat$x11 <- rep(scale(lat), rep(12,48))
tx.dat$x12 <- rep(scale(long), rep(12,48))
tx.dat$stacod <- rep(TXtemp$stacod[seq(1, nrow(TXtemp),by=360)] ,rep(12,48))

# plot all temp profiles for all stations
t <- seq(0.5, 11.5, 1)/12
postscript(file="TXtemp1.ps",height=2.8,width=4,pointsize=8,horizontal=F)
par(mgp=c(2,1,0), mar=c(3,3,1,1)+.1)
plot(tx.dat$y,tx.dat$x2,xlab="months",ylab="temperature (F)",
     cex=.8,type="n", axes=F,xlim=c(0,1),ylim=c(30,90))
for (j in unique(tx.dat$stacod)) {
  b <- tx.dat[tx.dat$stacod==j,]
  lines(b$x2,b$y,lwd=.5)
  points(b$x2,b$y,pch="x",cex=.8)
}
box()
axis(2,cex=.5)
axis(1,at=t,label=as.character(1:12),cex=0.5)
dev.off()

# plot as surfaces for 4 months
months <- c("January", "Feb", "Mar", "April", "May", "Jun",
            "July", "Aug", "Sep", "October", "Nov", "Dec")
postscript(file="TXtemp2.ps",height=3.5,width=4.4,pointsize=10,horizontal=F)
par(mfrow=c(2,2), mgp=c(0,0,0), mar=c(0,0,0,0)+0.1)
for (j in c(1,4,7,10)) {
  map("state","texas",interior=F,col=gray(.5))
  mtext(paste("                          ",months[j]),line=-2,cex=.8)
  a <- tx.dat[tx.dat$x2==unique(tx.dat$x2)[j],]
  i <- interp(a$long,a$lat,a$y)
  contour(i,add=T, cex=.8, lwd=.5)
  points(a$long,a$lat,cex=.2)
}
dev.off()


# fit SS ANOVA model
#source("tp.linear")
tx.ssanova <- ssr(y~x11+x12, data=tx.dat, 
   rk=list(tp(list(x11,x12)),
           periodic(x2),
           rk.prod(tp.linear(list(x11,x12)),periodic(x2)),
           rk.prod(tp(list(x11,x12)), periodic(x2))))
summary(tx.ssanova)


# prediction for selected stations
selsta <- c(tx.dat[tx.dat$x11==min(tx.dat$x11),7][1],
            tx.dat[tx.dat$x11==max(tx.dat$x11),7][1],
            tx.dat[tx.dat$x12==min(tx.dat$x12),7][1],
            tx.dat[tx.dat$x12==max(tx.dat$x12),7][1])
sellat <- sellong <- NULL
for (i in 1:4) {
  sellat <- c(sellat, tx.dat$x11[tx.dat$stacod==selsta[i]][1])
  sellong <- c(sellong, tx.dat$x12[tx.dat$stacod==selsta[i]][1])
}

grid <- data.frame(x11=rep(sellat,rep(40,4)), 
                   x12=rep(sellong ,rep(40,4)),
                   x2=rep(seq(0,1,len=40), 4))
tx.pred1 <- predict(tx.ssanova,grid,terms=c(0,1,1,1,0,1,1))

# plot of location effect
tmp1 <- data.frame(y=tx.pred1$fit,
                   month=grid$x2[1:40],
                   station=rep(c("Rio Grande City 3W","Stratford",
                     "El Paso WSO AP", "Marshall"),rep(40,4)))
tmp2 <- data.frame(y=tx.pred1$fit-1.96*tx.pred1$pstd,
                   month=grid$x2[1:40],
                   station=rep(c("Rio Grande City 3W","Stratford",
                     "El Paso WSO AP", "Marshall"),rep(40,4)))
tmp3 <- data.frame(y=tx.pred1$fit+1.96*tx.pred1$pstd,
                   month=grid$x2[1:40],
                   station=rep(c("Rio Grande City 3W","Stratford",
                     "El Paso WSO AP", "Marshall"),rep(40,4)))
tmp1$rowType <- rep("fit", nrow(tmp1))
tmp2$rowType <- rep("lb", nrow(tmp2))
tmp3$rowType <- rep("ub", nrow(tmp3))
all <- rbind(tmp1, tmp2, tmp3)
all$station <- ordered(all$station,
  level=c("El Paso WSO AP", "Marshall","Rio Grande City 3W","Stratford"))
trellis.device(postscript,file="TXtemp3.ps",width=4.6,height=3.5,
               horizontal=F)
xyplot(y~I(12*month+.5)|station, data=all, subscripts=T, groups=all$rowType,
       strip = function(...) strip.default(..., style = 1),
       strip.names=c(F,F), layout=c(2,2),
       par.strip.text = list(cex = 0.75),
       scales=list(cex=.75),
       xlab=list("month",cex=.75),
       ylab=list("temperature (F)",cex=.75),
       panel=function(x,y,subscripts,groups,...){
             data <- groups[subscripts]=="data"
             fit <- groups[subscripts]=="fit"
             lb <- groups[subscripts]=="lb"
             ub <- groups[subscripts]=="ub"
             panel.xyplot(x[fit],y[fit],type="n")
             panel.polygon(c(x[lb],rev(x[ub])),c(y[lb],rev(y[ub])),
             col=gray(0:8/8)[8],border=NA)
             llines(c(0,12.5),c(0,0),lty=3)
             llines(x[fit],y[fit])})
dev.off()

# prediction and plot of month effect
tx.pred2 <- predict(tx.ssanova,terms=c(0,0,0,0,1,1,1),pstd=F)
tx.dat$meff <- tx.pred2$fit
postscript(file="TXtemp4.ps",height=3.5,width=4.4,pointsize=10,horizontal=F)
par(mfrow=c(2,2), mgp=c(0,0,0))
for (j in c(1,4,7,10)) {
  if (j<=2) par(mar=c(1,0,0,0))
  if (j>2) par(mar=c(0,0,1,0))
  map("state","texas",interior=F,col=gray(.5))
  mtext(paste("                       ",months[j]),line=-1,cex=.8)
  a <- tx.dat[tx.dat$x2==unique(tx.dat$x2)[j],]
  i <- interp(a$long,a$lat,a$meff, linear=F)
  if (j==1) lv <- seq(-20,-15,by=1)
  if (j==4) lv <- seq(0,1.5,by=.3)
  if (j==7) lv <- seq(13,21,by=2)
  if (j==10) lv <- seq(0,2,by=.4)
  contour(i,add=T, cex=.8, lwd=.5, labcex=.8, levels=lv)
}
dev.off()

# fit a multiplicative model
g3.ini <- predict(ssr(y~1,rk=periodic(x2),data=tx.dat,spar="m"),
 terms=c(0,1), pstd=F)$fit
tx.nnr <- nnr(y~g1(x11,x12)+exp(g2(x11,x12))*g3(x2), 
 func=list(g1(x,z)~list(~x+z,tp(list(x,z))), 
           g2(x,z)~list(~x+z-1,tp(list(x,z))), 
           g3(x)~list(periodic(x))), 
 data=tx.dat,start=list(g1=mean(y),g2=0,g3=g3.ini)) 

ind <- seq(1,576,by=12)
i1 <- interp(tx.dat$long[ind],tx.dat$lat[ind],tx.nnr$funcFitted$g1[ind])
i2 <- interp(tx.dat$long[ind],tx.dat$lat[ind],tx.nnr$funcFitted$g2[ind])
postscript(file="TXtemp5.ps",height=2.2,width=4.4,pointsize=10,horizontal=F)
par(mfrow=c(1,2), mgp=c(0,0,0), mar=c(0,0,4,0))
map("state","texas",interior=F,col=gray(.5))
contour(i1,add=T,levels=seq(55,73,by=2),cex=0.6)
map("state","texas",interior=F,col=gray(.5))
contour(i2,add=T,levels=seq(-.2,.2,by=.04),cex=0.6)
dev.off()

# compare nnr with SS ANOVA
n <- length(y)
rss <- c(sum(tx.ssanova$resi**2),sum(tx.nnr$resi**2))
df <- c(tx.ssanova$df,tx.nnr$df$f)
gcv <- rss/(1-df/n)**2
aic <- n*log(rss)+2*df
bic <- n*log(rss)+log(n)*df
print(rbind(gcv,aic,bic))

# fit each station separately
tx.p.sep <- tx.p.sep.a <- NULL
stations <- unique(tx.dat$stacod)
for(i in 1:48){
 tmp <- ssr(y~1, rk=periodic(x2), spar="m", 
  subset=stacod==stations[i], data=tx.dat)
 tmpp <- predict(tmp, terms=c(0,1), pstd=F,
  newdata=data.frame(x2=seq(0,1,len=101)))$fit
 tmpp <- tmpp/diff(range(tmpp))
 maxinx <- min((1:101)[tmpp==max(tmpp)])
 tmpp.a <- tmpp
 if (maxinx>51) tmpp.a <- c(tmpp[(maxinx-50):101],tmpp[1:(maxinx-51)])
 if (maxinx<51) tmpp.a <- c(tmpp[(101+maxinx-50):101],tmpp[1:(101+maxinx-51)])
 tx.p.sep <- cbind(tx.p.sep, tmpp)
 tx.p.sep.a <- cbind(tx.p.sep.a, tmpp.a)
}

postscript(file="TXtemp6.ps",height=2,width=4.4,pointsize=10,horizontal=F)
par(mfrow=c(1,2), mgp=c(2,1,0), mar=c(3,3,2,2)+0.1)
plot(seq(0,1,len=101), tx.p.sep[,1], xlab=expression(x[2]), ylab="shape",
     type="n",ylim=range(tx.p.sep))
mtext("(a)")
for(i in 1:48) lines(seq(0,1,len=101), tx.p.sep[,i],lwd=.3)   
plot(seq(0,1,len=101), tx.p.sep.a[,1], xlab=expression(x[2]), ylab="shape",
     type="n",ylim=range(tx.p.sep.a))
mtext("(b)")
for(i in 1:48) lines(seq(0,1,len=101), tx.p.sep.a[,i],lwd=.3)   
dev.off()

postscript(file="TXtemp7.ps",height=2,width=4.4,pointsize=10,horizontal=F)
par(mfrow=c(1,2), mgp=c(2,1,0), mar=c(3,3,2,2)+0.1)
for (i in c(1,30)) {
 grid2 <- data.frame(x2=seq(0,1,len=40), x11=tx.dat$x11[i], x12=tx.dat$x12[i])
 tx.ssanova.p.f2 <- predict(tx.ssanova,grid2,terms=c(0,0,0,0,1,0,0),pstd=F)$fit
 tx.ssanova.p.f12 <- predict(tx.ssanova,grid2, terms=c(0,0,0,0,0,1,1),pstd=F)$fit
 plot(tx.ssanova.p.f2, tx.ssanova.p.f12, xlab=expression(f[2]), 
      ylab=expression(f[12]), type="n")
 if (i==1) mtext("Albany")
 if (i==30) mtext("Liberty")
 points(tx.ssanova.p.f2, tx.ssanova.p.f12,pch="o",cex=.5)
}
dev.off()





