# Code for ultrasound data

# 2words has 10 replicates and 9 frames
# cluster has 10 replicates and 10 frames
# schwa has 8 replicates and 11 frames

library(assist)
library(akima)
library(lattice)
rep <- c("A","B","C","D","E","F","G","H","I","J")
con <- c("2words","cluster","schwa")
data(ultrasound)

# plot of raw data
lgrid <- tgrid <- hgrid <- NULL
for (i in 1:3) {
  tmp <- ultrasound[ultrasound$env==i,]
  xo <- do.breaks(range(tmp$length),19)
  yo <- do.breaks(range(tmp$time),19)
  int <- interp(tmp$length,tmp$time,tmp$height,duplicate="mean",
                xo=xo, yo=yo)
  a <- expand.grid(lgrid=xo,tgrid=yo)
  lgrid <- c(lgrid, a$lgrid)
  tgrid <- c(tgrid, a$tgrid)
  hgrid <- c(hgrid, as.vector(int$z))
}
lthgrid <- data.frame(lgrid=lgrid, tgrid=tgrid, hgrid=hgrid, 
 env=rep(con, rep(400,3)))

postscript(file="ultra1.ps",height=2.5*1.5,width=4.45*1.5,horizontal=F,
  pointsize=10)
wireframe(hgrid~lgrid*tgrid|env, data=lthgrid, 
 screen=list(z=25,x=-75), 
 xlab=list(label="length (mm)", rot=10, cex=.7),
 ylab=list(label="time (ms)", rot=-45, cex=.7),
 zlab=list(label="height (mm)", rot=90, cex=.7), 
 scales=list(arrows=F, cex=.45, col="black", tck=1))
dev.off()


# First question: how tongue shapes differ among environments
# one discrete and one continuous variable
ultrasound$y <- ultrasound$height
ultrasound$x1 <- ultrasound$env
ultrasound$x2 <- ident(ultrasound$length)

# linear spline for x2 
ultra.el.l.fit <- ssr(y~1, data=ultrasound, subset=ultrasound$time==60,
                      rk=list(shrink1(x1), 
                              linear(x2),
                              rk.prod(shrink1(x1),linear(x2))))

# cubic spline for x2
ultra.el.c.fit <- ssr(y~x2, data=ultrasound, subset=ultrasound$time==60,
                      rk=list(shrink1(x1), 
                              cubic(x2),
                              rk.prod(shrink1(x1),kron(x2-.5)),
                              rk.prod(shrink1(x1),cubic(x2))))
summary(ultra.el.c.fit)

# same smoothing parameter for two interactions
ultra.el.c.fit1 <- ssr(y~x2, data=ultrasound, subset=ultrasound$time==60,
                       rk=list(shrink1(x1),
                               cubic(x2),
                               rk.prod(shrink1(x1),kron(x2-.5))+
                               rk.prod(shrink1(x1),cubic(x2))))
summary(ultra.el.c.fit1)

# compute fits and posterior standard deviations
grid <- seq(0,1,len=40)

# fits for each environment
ultra.el.c.pred1 <- predict(ultra.el.c.fit,
               newdata=expand.grid(x2=grid,x1=as.factor(1:3)),
               terms=c(1,1,1,1,1,1), pstd=T)

# mean curve
ultra.el.c.pred2 <- predict(ultra.el.c.fit,
               newdata=expand.grid(x2=grid,x1=as.factor(1)),
               terms=c(1,1,0,1,0,0))

ultra1 <- ultrasound[ultrasound$time==60,]
ggrid <- grid*diff(range(ultra1$length))+min(ultra1$length)
yr <- range(ultra.el.c.pred1$fit,ultra1$height)

postscript(file="ultra2.ps",height=2.2,width=4.6,pointsize=10,horizontal=F)
par(mfrow=c(1,3),oma=c(0,2,0,0))
for (i in 1:3) {
  if (i==1) par(mgp=c(2,1,0), mar=c(3,3,3,0))
  if (i==2) par(mgp=c(2,1,0),mar=c(3,2,3,1))
  if (i==3) par(mgp=c(2,1,0),mar=c(3,1,3,2))
  sel <- ((i-1)*length(grid)+1):(i*length(grid))
  if (i==1) 
   plot(ggrid,ultra.el.c.pred1$fit[sel],xlab="length (mm)", 
     ylab="height (mm)",axes=F,type="n",ylim=yr)
  if (i>1) 
   plot(ggrid,ultra.el.c.pred1$fit[sel],xlab="length (mm)", 
     ylab="",axes=F,type="n",ylim=yr)
  polygon(c(ggrid,rev(ggrid)),
    c(ultra.el.c.pred1$fit[sel]-1.96*ultra.el.c.pred1$pstd[sel],
      rev(ultra.el.c.pred1$fit[sel]+1.96*ultra.el.c.pred1$pstd[sel])), 
  col="lightgray",border=NA)            
  lines(ggrid,ultra.el.c.pred1$fit[sel])
  lines(ggrid,ultra.el.c.pred2$fit,lwd=2)
  tmp <- ultra1[ultra1$env==i,]
  points(tmp$length, tmp$height, pch="o",cex=.4)
  box()
  axis(1)
  if (i==1) axis(2)
  mtext(con[i])  
}
dev.off()

# interactions
ultra.el.c.pred3 <- predict(ultra.el.c.fit,
    newdata=expand.grid(x2=grid,x1=as.factor(1:3)),
    terms=c(0,0,0,0,1,1))
yr <- range(ultra.el.c.pred3$fit-1.96*ultra.el.c.pred3$pstd,
            ultra.el.c.pred3$fit+1.96*ultra.el.c.pred3$pstd)
postscript(file="ultra3.ps",height=2.2,width=4.6,pointsize=10,horizontal=F)
par(mfrow=c(1,3),oma=c(0,2,0,0))
for (i in 1:3) {
  if (i==1) par(mgp=c(2,1,0),mar=c(3,3,3,0))
  if (i==2) par(mgp=c(2,1,0),mar=c(3,2,3,1))
  if (i==3) par(mgp=c(2,1,0),mar=c(3,1,3,2))
  sel <- ((i-1)*length(grid)+1):(i*length(grid))
  if (i==1) 
    plot(ggrid,ultra.el.c.pred3$fit[sel],xlab="length (mm)", 
      ylab="height (mm)",axes=F,type="n",ylim=yr)
  if (i>1) 
    plot(ggrid,ultra.el.c.pred3$fit[sel],xlab="length (mm)", 
      ylab="",axes=F,type="n",ylim=yr)
  polygon(c(ggrid,rev(ggrid)),
    c(ultra.el.c.pred3$fit[sel]-1.96*ultra.el.c.pred3$pstd[sel],
      rev(ultra.el.c.pred3$fit[sel]+1.96*ultra.el.c.pred3$pstd[sel])),
    col="lightgray",border=NA)
  lines(ggrid,ultra.el.c.pred3$fit[sel])
  box()
  axis(1)
  if (i==1) axis(2)
  mtext(con[i])  
  abline(0,0,lty=2)    
}
dev.off()


# compute effects
ultra.el.c.pred4 <- predict(ultra.el.c.fit,
    newdata=expand.grid(x2=grid,x1=as.factor(1:3)),
    terms=c(0,0,1,0,1,1))
yr <- range(ultra.el.c.pred4$fit-1.96*ultra.el.c.pred4$pstd,
            ultra.el.c.pred4$fit+1.96*ultra.el.c.pred4$pstd)
postscript(file="ultra4.ps",height=2.2,width=4.6,pointsize=10,horizontal=F)
par(mfrow=c(1,3),oma=c(0,2,0,0))
for (i in 1:3) {
  if (i==1) par(mgp=c(2,1,0),mar=c(3,3,3,0))
  if (i==2) par(mgp=c(2,1,0),mar=c(3,2,3,1))
  if (i==3) par(mgp=c(2,1,0),mar=c(3,1,3,2))
  sel <- ((i-1)*length(grid)+1):(i*length(grid))
  if (i==1) 
    plot(ggrid,ultra.el.c.pred4$fit[sel],xlab="length (mm)", 
      ylab="height (mm)",axes=F,type="n",ylim=yr)
  if (i>1) 
    plot(ggrid,ultra.el.c.pred4$fit[sel],xlab="length (mm)", 
      ylab="",axes=F,type="n",ylim=yr)
  polygon(c(ggrid,rev(ggrid)),
    c(ultra.el.c.pred4$fit[sel]-1.96*ultra.el.c.pred4$pstd[sel],
      rev(ultra.el.c.pred4$fit[sel]+1.96*ultra.el.c.pred4$pstd[sel])),
    col="lightgray",border=NA)
  lines(ggrid,ultra.el.c.pred4$fit[sel])
  box()
  axis(1)
  if (i==1) axis(2)
  mtext(con[i])  
  abline(0,0,lty=2)    
}
dev.off()



# Second question: how tongue shapes change over time
# two continuous variables
ultrasound$x3 <- ident(ultrasound$time)
ngrid <- 20
grid <- seq(0,1,len=ngrid)
gridx2 <- min(ultrasound$length)+grid*diff(range(ultrasound$length))
gridx3 <- min(ultrasound$time)+grid*diff(range(ultrasound$time))
a <- expand.grid(lgrid=gridx2,tgrid=gridx3)

# fit tensor product of linear splines
ultra.lt.l.fit <- list(NULL,NULL,NULL)
lgrid <- tgrid <- hgrid <- NULL
for (i in 1:3) {
 fit <- ssr(height~1, data=ultrasound, subset=ultrasound$env==i,
            rk=list(linear(x2),
                    linear(x3),
                    rk.prod(linear(x2),linear(x3))))
 ultra.lt.l.fit[[i]] <- fit
 pred <- predict(fit,newdata=expand.grid(x2=grid,x3=grid))
 lgrid <- c(lgrid, a$lgrid)
 tgrid <- c(tgrid, a$tgrid)
 hgrid <- c(hgrid, pred$fit)
}
lthgrid <- data.frame(lgrid=lgrid, tgrid=tgrid, hgrid=hgrid,
 env=rep(con, rep(ngrid**2,3)))
   
postscript(file="ultra5.1.ps",height=2.5*1.5,width=4.45*1.5,horizontal=F,
  pointsize=10)
wireframe(hgrid~lgrid*tgrid|env, data=lthgrid,
 screen=list(z=25,x=-75),
 xlab=list(label="length (mm)", rot=10, cex=.7),
 ylab=list(label="time (ms)", rot=-45, cex=.7),
 zlab=list(label="height (mm)", rot=90, cex=.7),
 scales=list(arrows=F, cex=.45, col="black", tck=1))
dev.off()



# fit tensor product of cubic splines
ultra.lt.c.fit <- list(NULL,NULL,NULL)
lgrid <- tgrid <- hgrid <- NULL
for (i in 1:3) {
 fit <- ssr(height~x2+x3+x2*x3, data=ultrasound,
  subset=ultrasound$env==i,
  rk=list(cubic(x2),
          cubic(x3),
          rk.prod(kron(x2-.5),cubic(x3)),
          rk.prod(cubic(x2),kron(x3-.5)),
          rk.prod(cubic(x2),cubic(x3))))
 ultra.lt.c.fit[[i]] <- fit
 pred <- predict(fit,newdata=expand.grid(x2=grid,x3=grid))
 lgrid <- c(lgrid, a$lgrid)
 tgrid <- c(tgrid, a$tgrid)
 hgrid <- c(hgrid, pred$fit)
}
lthgrid <- data.frame(lgrid=lgrid, tgrid=tgrid, hgrid=hgrid,
 env=rep(con, rep(ngrid**2,3)))

postscript(file="ultra5.ps",height=2.5*1.5,width=4.45*1.5,horizontal=F,
  pointsize=10)
wireframe(hgrid~lgrid*tgrid|env, data=lthgrid,
 screen=list(z=25,x=-75),
 xlab=list(label="length (mm)", rot=10, cex=.7),
 ylab=list(label="time (ms)", rot=-45, cex=.7),
 zlab=list(label="height (mm)", rot=90, cex=.7),
 scales=list(arrows=F, cex=.45, col="black", tck=1))
dev.off()


# check time effects
postscript(file="ultra6.ps",height=2.2,width=4.6,pointsize=10,horizontal=F)
par(mfrow=c(1,3),oma=c(0,2,0,0))

for (i in 1:3) {
  if (i==1) par(mgp=c(2,1,0),mar=c(3,3,3,0))
  if (i==2) par(mgp=c(2,1,0),mar=c(3,2,3,1))
  if (i==3) par(mgp=c(2,1,0),mar=c(3,1,3,2))

  pred <- predict(ultra.lt.c.fit[[i]], 
    newdata=expand.grid(x2=grid,x3=grid),
    term=c(0,0,1,1,0,1,1,1,1),pstd=T)
  d <- matrix(pred$fit[1:ngrid**2],ngrid,ngrid)
  lb <- d-1.96*matrix(pred$pstd[1:ngrid**2],ngrid,ngrid)
  ub <- d+1.96*matrix(pred$pstd[1:ngrid**2],ngrid,ngrid)
  z <- 0*d
  z[lb>0] <- 1  # significantly larger
  z[ub<0] <- -1 # significantly smaller

  if (i==1) image(gridx2, gridx3, z, col=gray(c(.9,1,.8)), 
      xlab="length (mm)", ylab="time (ms)", axes=F)
  if (i>1) image(gridx2, gridx3, z, col=gray(c(.9,1,.8)), 
      xlab="length (mm)", ylab="", axes=F)
  if (i==1) lv <- c(-2,0,2,4,6,8)
  if (i==2) lv <- c(-7,-5,-3,0,5,10) 
  if (i==3) lv <- c(-4,-2,0,2,4,6)
  contour(gridx2, gridx3, d, add=T, levels=lv, labcex=.6)
  box()
  axis(1)
  if (i==1) axis(2)
  mtext(con[i])
}
dev.off()


# Question 3: how the changes of tongue shapes over time
# differ among different environments
# one discrete and 2 continuous variables
# R^a x W_2^1 x W_2^1 
# may need to run separatly due to large memory required
ultra.elt.c.fit <- ssr(height~I(x2-.5)+I(x3-.5)+I((x2-.5)*(x3-.5)), 
             data=ultrasound, 
             rk=list(shrink1(x1),
                     cubic(x2),
                     cubic(x3),
                     rk.prod(shrink1(x1),kron(x2-.5)),
                     rk.prod(shrink1(x1), cubic(x2)),
                     rk.prod(shrink1(x1),kron(x3-.5)),
                     rk.prod(shrink1(x1),cubic(x3)),
                     rk.prod(cubic(x2),kron(x3-.5)),
                     rk.prod(kron(x2-.5),cubic(x3)),
                     rk.prod(cubic(x2),cubic(x3)),
                     rk.prod(shrink1(x1),kron(x2-.5),kron(x3-.5)),
                     rk.prod(shrink1(x1),kron(x2-.5),cubic(x3)),
                     rk.prod(shrink1(x1),cubic(x2),kron(x3-.5)),
                     rk.prod(shrink1(x1),cubic(x2),cubic(x3))))

ngrid <- 20
grid <- seq(0,1,len=ngrid)
gridx2 <- min(ultrasound$length)+grid*diff(range(ultrasound$length))
gridx3 <- min(ultrasound$time)+grid*diff(range(ultrasound$time))
allgrid <- expand.grid(x1=as.factor(1:3),x2=grid,x3=grid)
pred <- predict(ultra.elt.c.fit, newdata=allgrid)
allgrid <- expand.grid(x1=con,x2=grid,x3=grid)
lthgrid <- data.frame(lgrid=allgrid$x2, tgrid=allgrid$x3,
 hgrid=pred$fit, env=allgrid$x1)

postscript(file="ultra7.ps",height=2.5*1.5,width=4.45*1.5,horizontal=F,
  pointsize=10)
wireframe(hgrid~lgrid*tgrid|env, data=lthgrid,
 screen=list(z=25,x=-75),
 xlab=list(label="length (mm)", rot=10, cex=.7),
 ylab=list(label="time (ms)", rot=-45, cex=.7),
 zlab=list(label="height (mm)", rot=90, cex=.7),
 scales=list(arrows=F, cex=.45, col="black", tck=1))
dev.off()


ngrid <- 25
grid <- seq(0,1,len=ngrid)
gridx2 <- min(ultrasound$length)+grid*diff(range(ultrasound$length))
gridx3 <- min(ultrasound$time)+grid*diff(range(ultrasound$time))
allgrid <- expand.grid(x1=as.factor(1:3),x2=grid,x3=grid)
ultra.elt.c.eff <- predict(ultra.elt.c.fit, newdata=allgrid, pstd=T,
    terms=c(0,0,0,0,1,0,0,1,1,1,1,0,0,0,1,1,1,1))


postscript(file="ultra8.ps",height=2.2,width=4.6,pointsize=10,horizontal=F)
par(mfrow=c(1,3),oma=c(0,2,0,0))

for (i in 1:3) {
  if (i==1) par(mgp=c(2,1,0),mar=c(3,3,3,0))
  if (i==2) par(mgp=c(2,1,0),mar=c(3,2,3,1))
  if (i==3) par(mgp=c(2,1,0),mar=c(3,1,3,2))

  d <- matrix(ultra.elt.c.eff$fit[allgrid==i],ngrid,ngrid)
  lb <- d-1.96*matrix(ultra.elt.c.eff$pstd[allgrid==i],ngrid,ngrid)
  ub <- d+1.96*matrix(ultra.elt.c.eff$pstd[allgrid==i],ngrid,ngrid)
  z <- 0*d
  z[lb>0] <- 1  # significantly larger
  z[ub<0] <- -1 # significantly smaller
  if (i==1) image(gridx2, gridx3, z, col=gray(c(.9,1,.8)), 
      xlab="length (mm)", ylab="time (ms)", axes=F)
  if (i>1) image(gridx2, gridx3, z, col=gray(c(.9,1,.8)), 
      xlab="length (mm)", ylab="", axes=F)
  if (i==1) lv <- c(-2,-1,0,2,4,6)
  if (i==2) lv <- c(-5,-3,-1,0,1,3) 
  if (i==3) lv <- c(-4,-2,-1,0,1,3)
  contour(gridx2, gridx3, d, add=T, levels=lv, labcex=.6)
  box()
  axis(1)
  if (i==1) axis(2)
  mtext(con[i])
}
dev.off()

