# Code for WESDR data

library(assist)
data(wesdr)
attach(wesdr)
norm.cen <- function(x) sqrt(sum((x-mean(x))**2))

y <- prg
x1 <- (dur-min(dur))/diff(range(dur))
x2 <- (gly-min(gly))/diff(range(gly))
x3 <- (bmi-min(bmi))/diff(range(bmi))

# fit an SS ANOVA model with all 2-way interactions
wesdr.fit1 <- ssr(y~I(x1-.5)+I(x2-.5)+I(x3-.5)+
 I((x1-.5)*(x2-.5))+I((x1-.5)*(x3-.5))+I((x2-.5)*(x3-.5)), 
 rk=list(cubic(x1), 
         cubic(x2), 
         cubic(x3), 
         rk.prod(kron(x1-.5), cubic(x2)),  
         rk.prod(kron(x2-.5), cubic(x1)),  
         rk.prod(cubic(x1), cubic(x2)),
         rk.prod(kron(x1-.5), cubic(x3)), 
         rk.prod(kron(x3-.5), cubic(x1)),
         rk.prod(cubic(x1), cubic(x3)),
         rk.prod(kron(x2-.5), cubic(x3)), 
         rk.prod(kron(x3-.5),cubic(x2)),
         rk.prod(cubic(x2), cubic(x3))),
         family="binary", spar="u", varht=1)
summary(wesdr.fit1)

# compute norms for all components (centered)
comp.est1 <- predict(wesdr.fit1, terms=diag(rep(1,19))[-1,], pstd=F)
comp.norm1 <- apply(comp.est1$fit, 2, norm.cen)
print(round(comp.norm1,2))

# fit without nonparametric interactions between dur and gly
wesdr.fit2 <- update(wesdr.fit1, 
 rk=list(cubic(x1),
         cubic(x2),
         cubic(x3),
         rk.prod(kron(x1-.5), cubic(x3)),
         rk.prod(kron(x3-.5), cubic(x1)),
         rk.prod(cubic(x1), cubic(x3)),
         rk.prod(kron(x2-.5), cubic(x3)),
         rk.prod(kron(x3-.5),cubic(x2)),
         rk.prod(cubic(x2), cubic(x3))))
summary(wesdr.fit2)
comp.est2 <- predict(wesdr.fit2, terms=diag(rep(1,16))[-1,], pstd=F)
comp.norm2 <- apply(comp.est2$fit, 2, norm.cen)
print(round(comp.norm2,2))

# check 95% Bayesian confidence intervals
int.dur.bmi <- predict(wesdr.fit2, terms=c(rep(0,10),rep(1,3),rep(0,3)))
mean((int.dur.bmi$fit-1.96*int.dur.bmi$pstd>0)|(int.dur.bmi$fit+1.96*int.dur.bmi$pstd<0))

int.gly.bmi <- predict(wesdr.fit2, terms=c(rep(0,13),rep(1,3)))
mean((int.gly.bmi$fit-1.96*int.gly.bmi$pstd>0)|(int.gly.bmi$fit+1.96*int.gly.bmi$pstd<0))

# fit without nonparametric interactions 
wesdr.fit3 <- update(wesdr.fit2,
 rk=list(cubic(x1), cubic(x2), cubic(x3)))
summary(wesdr.fit3)
comp.est3 <- predict(wesdr.fit3, terms=diag(rep(1,10))[-1,], pstd=F)
comp.norm3 <- apply(comp.est3$fit, 2, norm.cen)
print(round(comp.norm3,2))

# fit without nonparametric main effect of gly
wesdr.fit4 <- update(wesdr.fit3, rk=list(cubic(x1), cubic(x3)))
summary(wesdr.fit4)

postscript(file="wesdr.ps",height=2.1,width=4.4,horizontal=F,pointsize=10)
par(mfrow=c(1,3), mgp=c(2,1,0), mar=c(3,3,3,1)+0.1)

grid <- expand.grid(x1=seq(0,1,len=40), x2=median(x2), x3=median(x3))
p <- predict(wesdr.fit4, grid)
xx <- grid$x1*diff(range(dur))+min(dur)
plot(xx, alogit(p$fit), type="n", ylim=c(0,1), xlab="duration (yr)", 
 ylab="probability",cex=.8) 
polygon(c(xx, rev(xx)),
        alogit(c(p$fit-1.96*p$pstd, rev(p$fit+1.96*p$pstd))),
        col=gray(0:8/8)[8], border=NA)
lines(xx, alogit(p$fit))
rug(dur[prg==0])
rug(dur[prg==1],side=3)
mtext("(a)", cex=.9)

grid <- expand.grid(x2=seq(0,1,len=40), x1=median(x1), x3=median(x3))
p <- predict(wesdr.fit4, grid)
xx <- grid$x2*diff(range(gly))+min(gly)
plot(xx, alogit(p$fit), type="n", ylim=c(0,1), xlab="gly. hemoglobin",
 ylab="probability",cex=.8)
polygon(c(xx,rev(xx)),
        alogit(c(p$fit-1.96*p$pstd, rev(p$fit+1.96*p$pstd))),
        col=gray(0:8/8)[8], border=NA)
lines(xx, alogit(p$fit))
rug(gly[prg==0])
rug(gly[prg==1],side=3)
mtext("(b)", cex=.9)

grid <- expand.grid(x3=seq(0,1,len=40), x1=median(x1), x2=median(x2))
p <- predict(wesdr.fit4, grid)
xx <- grid$x3*diff(range(bmi))+min(bmi)
plot(xx, alogit(p$fit), type="n", ylim=c(0,1), 
 xlab=expression(paste("body mass index (kg/",m^2,")")),
 ylab="probability",cex=.8)
polygon(c(xx,rev(xx)),
        alogit(c(p$fit-1.96*p$pstd, rev(p$fit+1.96*p$pstd))),
        col=gray(0:8/8)[8], border=NA)
lines(xx, alogit(p$fit))
rug(bmi[prg==0])
rug(bmi[prg==1],side=3)
mtext("(c)", cex=.9)

dev.off()

